# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Helper module with ImportMockingTestCase class
"""

import sys
import gc
import types

from unittest import TestCase


class _ImportProhibitorHook(object):

    def __init__(self):
        self.prohibited = set()

    def disallow(self, fullname):
        self.prohibited.add(fullname)

    def find_module(self, fullname, path=None):
        if fullname in self.prohibited:
            return self

    def load_module(self, fullname):
        raise ImportError("Importing module %s is prohibited" % (fullname,))


class ImportMockingTestCase(TestCase):

    def setUp(self):
        super(ImportMockingTestCase, self).setUp()
        self._hook = _ImportProhibitorHook()
        self._hidden_modules = {}

    def tearDown(self):
        self._restore_hidden()
        super(ImportMockingTestCase, self).tearDown()

    def prohibit_importing(self, fullname):
        """
        Disallow importing of module `fullname'.
        During the rest of the test case any attempts to import
        this module will raise ImportError.
        """
        self._hook.disallow(fullname)

    def _get_referring_modules(self, obj):
        """
        Find all modules that *directly* refer to object `obj'
        """
        for ref_obj in gc.get_referrers(obj):
            if isinstance(ref_obj, dict) and '__name__' in ref_obj:
                for mod in gc.get_referrers(ref_obj):
                    if isinstance(mod, types.ModuleType):
                        yield mod.__name__

    def mock_imports(self, reload_list=None):
        """
        Make prohibit_importing() work by hiding and importing
        again all the modules that depended on an imported
        prohibited module.

        This does _NOT_ work 100% reliably as it only finds modules that
        directly imported one of the prohibited modules AND it depends
        on being able to safely reimport them. If autodetection fails
        pass a module, or a list of modules to reimport.

        The side effects of this function last until the end of the test
        case, in other words, until tearDown() is implicitly called.
        """
        to_reload = set()
        if reload_list is not None:
            if isinstance(reload_list, basestring):
                reload_list = [reload_list]
            to_reload.update(reload_list)
        to_hide = set()
        to_hide.update(to_reload)

        # For all the things we want to disallow
        for fullname in self._hook.prohibited:
            # Modules not yet loaded do not require any actions
            if fullname not in sys.modules:
                continue
            # Loaded modules must be hidden
            to_hide.add(fullname)
            # And all of their dependencies must be reloaded
            module = sys.modules[fullname]
            for related_fullname in self._get_referring_modules(module):
                if hasattr(sys.modules[related_fullname],
                        '__inhibit_protect__'):
                    continue
                to_hide.add(related_fullname)
                to_reload.add(related_fullname)
        # Install our meta-import hook
        sys.meta_path.append(self._hook)
        # Hide modules
        for fullname in to_hide:
            self._hide(fullname)
        # Reload modules
        for fullname in to_reload:
            self._reload(fullname)

    def _hide(self, fullname):
        self._hidden_modules[fullname] = sys.modules[fullname]
        del sys.modules[fullname]

    def _reload(self, fullname):
        # Only reload if it is not loaded already
        # This might seem crazy but it is required because
        # we reload a sequence of modules that might import
        # things themselves.
        # Example:
        #   1) to_reload: a, b; to_hide: a, b
        #      original a and b are now hidden
        #   2) _reload('a')
        #   3) we __import__('a')
        #   4) a imports b
        #   5) _reload('b')
        #   6) 'b' is already imported so we don't do anything about it
        if fullname not in sys.modules:
            __import__(fullname, fromlist=[''])

    def _restore_hidden(self):
        if self._hook in sys.meta_path:
            sys.meta_path.remove(self._hook)
        sys.modules.update(self._hidden_modules)



"""
Package with all tests for dashboard_app
"""

import logging
import unittest

TEST_MODULES = [
    'models.attachment',
    'models.bundle',
    'models.bundle_deserialization_error',
    'models.bundle_stream',
    'models.data_report',
    'models.hw_device',
    'models.named_attribute',
    'models.sw_package',
    'models.test',
    'models.test_case',
    'models.test_result',
    'models.test_run',
    'other.csrf',
    'other.dashboard_api',
    'other.dataview',
    'other.deserialization',
    'other.login',
    'other.test_client',
    'regressions.LP658917',
    'views.bundle_stream_list_view',
    'views.test_run_detail_view',
    'views.test_run_list_view',
    'views.redirects',
]

def load_tests_from_submodules(_locals):
    """
    Load all test classes from sub-modules as if they were here locally.

    This makes django test dispatcher work correctly and allows users to
    use the optional test identifier. The identifier has this format:
        Application.TestClass[.test_method]
    """
    for name in TEST_MODULES:
        module_name = 'dashboard_app.tests.' + name
        try:
            module = __import__(module_name, fromlist=[''])
        except ImportError:
            logging.exception("Unable to import test module %s", module_name)
            raise
        else:
            for attr in dir(module):
                obj = getattr(module, attr)
                if isinstance(obj, type) and issubclass(obj, unittest.TestCase):
                    _locals[attr] = obj

load_tests_from_submodules(locals())



# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Unit tests of the Dashboard application
"""
import datetime
import decimal

from django_testscenarios.ubertest import (
    TestCase,
    TestCaseWithScenarios,
    TransactionTestCase,
)
from django.contrib.auth.models import User
from linaro_dashboard_bundle.errors import DocumentFormatError
from linaro_json.schema import ValidationError
from linaro_json.extensions import datetime_extension


from dashboard_app.tests import fixtures
from dashboard_app.models import (
        Attachment,
        Bundle,
        BundleDeserializationError,
        BundleStream,
        HardwareDevice,
        NamedAttribute,
        SoftwarePackage,
        SoftwareSource,
        Test,
        TestCase as TestCaseModel,
        TestResult,
        TestRun,
        )
from dashboard_app.helpers import (
    BundleDeserializer,
    IBundleFormatImporter,
    BundleFormatImporter_1_0,
    BundleFormatImporter_1_1,
)



class IBundleFormatImporterTests(TestCase):

    def test_import_document_is_not_implemented(self):
        importer = IBundleFormatImporter()
        self.assertRaises(NotImplementedError,
                          importer.import_document, None, None)


class TestHelper(object):

    def getUniqueString(self, prefix=None, max_length=None):
        value = super(TestHelper, self).getUniqueString(prefix)
        if max_length is not None:
            if len(value) >= max_length:
                value = super(TestHelper, self).getUniqueString("short")
                if len(value) >= max_length:
                    raise ValueError("Unable to satisfy request for random string with max_length=%d" % max_length)
        return value

    def getUniqueStringForField(self, model, field_name):
        return self.getUniqueString(max_length=model._meta.get_field_by_name(field_name)[0].max_length)


class BundleFormatImporter_1_1Tests(TestHelper, TestCaseWithScenarios):

    scenarios = [
        ('with_commit_timestamp', {
            "commit_timestamp": datetime.datetime.now(),
        }),
        ('without_commit_timestamp', {
            "commit_timestamp": None
        })
    ]

    def c_getUniqueSoftwareSource(self):
        source = {
            "project_name": self.getUniqueStringForField(SoftwareSource, "project_name"),
            "branch_url": self.getUniqueStringForField(SoftwareSource, "branch_url"),
            "branch_vcs": self.getUniqueStringForField(SoftwareSource, "branch_vcs"),
            "branch_revision": self.getUniqueStringForField(SoftwareSource, "branch_revision"),
        }
        if self.commit_timestamp is not None:
            source["commit_timestamp"] = datetime_extension.to_json(self.commit_timestamp)
        return source

    def s_getUniqueTest(self):
        return Test.objects.create(
            test_id = self.getUniqueStringForField(Test, "test_id")
        )

    def s_getUniqueBundle(self):
        return Bundle.objects.create(
            bundle_stream = self.s_getUniqueBundleStream()
        )

    def s_getUniqueBundleStream(self):
        return BundleStream.objects.create(
            user = User.objects.create(username="legacy"),
            group = None
        )

    def s_getUniqueTestRun(self):
        return TestRun.objects.create(
            test = self.s_getUniqueTest(),
            bundle = self.s_getUniqueBundle(),
            analyzer_assigned_date = datetime.datetime.now(),
            analyzer_assigned_uuid = self.getUniqueStringForField(TestRun, "analyzer_assigned_uuid"),
        )

    def test_import_sources(self):
        c_test_run = {
            "software_context": {
                "sources": [
                    self.c_getUniqueSoftwareSource()
                    for i in range(3)
                ]
            }
        }
        s_test_run = self.s_getUniqueTestRun()
        importer = BundleFormatImporter_1_1()
        importer._import_sources(c_test_run, s_test_run)
        for c_source in c_test_run['software_context']['sources']:
            filter = dict(
                project_name = c_source['project_name'],
                branch_url = c_source['branch_url'],
                branch_vcs = c_source['branch_vcs'],
                branch_revision = str(c_source['branch_revision']),
                commit_timestamp = (
                    datetime_extension.from_json(
                        c_source["commit_timestamp"])
                    if "commit_timestamp" in c_source
                    else None)
            )
            s_source = SoftwareSource.objects.get(**filter)
            self.assertTrue(s_source is not None)
            self.assertTrue(s_source.pk is not None)
            self.assertTrue(s_source in s_test_run.sources.all())


class BundleBuilderMixin(object):
    """
    Helper mix-in for constructing bundle contents for unit testing
    """

    def getUniqueSoftwarePackage(self):
        return {
            "name": self.getUniqueStringForField(SoftwarePackage, "name"),
            "version": self.getUniqueStringForField(SoftwarePackage, "version")
        }

    def getUniqueSoftwareImage(self):
        return {
            "desc": self.getUniqueStringForField(TestRun, "sw_image_desc")
        }

    def getUniqueSoftwareContext(self, num_packages=None):
        if num_packages is None:
            num_packages = 5 # Arbitrary choice
        return {
            "sw_image": self.getUniqueSoftwareImage,
            "packages": [
                self.getUniqueSoftwarePackage() for i in range(num_packages)]
        }

    def getUniqueAttributes(self):
        attrs = {}
        for i in range(3):
            attrs[self.getUniqueStringForField(NamedAttribute, "name")] = \
                    self.getUniqueStringForField(NamedAttribute, "value")
        for i in range(3):
            attrs[self.getUniqueStringForField(NamedAttribute, "name")] = self.getUniqueInteger()
        return attrs

    def getUniqueHardwareDevice(self):
        return {
            "device_type": self.getUniqueStringForField(HardwareDevice, "device_type"),
            "description": self.getUniqueStringForField(HardwareDevice, "description"),
            "attributes": self.getUniqueAttributes(),
        }

    def getUniqueHardwareContext(self, num_devices=None):
        if num_devices is None:
            num_devices = 5 # Another arbitrary choice
        return {
            "devices": [
                self.getUniqueHardwareDevice() for i in range(num_devices)]
        }


class BundleFormatImporter_1_0Tests(
    TestHelper,
    TestCase,
    BundleBuilderMixin):

    def setUp(self):
        super(BundleFormatImporter_1_0Tests, self).setUp()
        self.importer = BundleFormatImporter_1_0()

    def test_get_sw_context_with_context(self):
        sw_context = self.getUniqueSoftwareContext()
        test_run = {"sw_context": sw_context}
        retval = self.importer._get_sw_context(test_run)
        self.assertEqual(retval, sw_context)

    def test_get_sw_context_without_context(self):
        test_run = {} # empty test run
        retval = self.importer._get_sw_context(test_run)
        self.assertEqual(retval, {})

    def test_get_hw_context_with_context(self):
        hw_context = self.getUniqueHardwareContext()
        test_run = {"hw_context": hw_context}
        retval = self.importer._get_hw_context(test_run)
        self.assertEqual(retval, hw_context)

    def test_get_hw_context_without_context(self):
        test_run = {} # empty test run
        retval = self.importer._get_hw_context(test_run)
        self.assertEqual(retval, {})

    def test_translate_result_string(self):
        self.assertEqual(
            self.importer._translate_result_string("pass"),
            TestResult.RESULT_PASS)
        self.assertEqual(
            self.importer._translate_result_string("fail"),
            TestResult.RESULT_FAIL)
        self.assertEqual(
            self.importer._translate_result_string("skip"),
            TestResult.RESULT_SKIP)
        self.assertEqual(
            self.importer._translate_result_string("unknown"),
            TestResult.RESULT_UNKNOWN)

    def test_translate_bogus_result_string(self):
        self.assertRaises(KeyError,
                          self.importer._translate_result_string,
                          "impossible result")


class BundleDeserializerSuccessTests(TestCaseWithScenarios):

    json_text = '''
    {
        "format": "Dashboard Bundle Format 1.0",
        "test_runs": [
            {
                "test_id": "some_test_id",
                "analyzer_assigned_uuid": "1ab86b36-c23d-11df-a81b-002163936223",
                "analyzer_assigned_date": "2010-12-31T23:59:59Z",
                "time_check_performed": true,
                "test_results": [
                {
                    "test_case_id": "some_test_case_id",
                    "result": "unknown",
                    "measurement": 1000.3,
                    "units": "bogomips",
                    "timestamp": "2010-09-17T16:34:21Z",
                    "duration": "1d 1s 1us",
                    "message": "text message",
                    "log_filename": "file.txt",
                    "log_lineno": 15,
                    "attributes": {
                        "attr1": "value1",
                        "attr2": "value2"
                    }
                },
                {
                    "test_case_id": "some_test_case_id",
                    "result": "unknown"
                }
                ],
                "sw_context": {
                    "packages": [
                        {"name": "pkg1", "version": "1.0"},
                        {"name": "pkg1", "version": "1.0"},
                        {"name": "pkg2", "version": "0.5"}
                    ],
                    "sw_image": {
                        "desc": "Ubuntu 10.10"
                    }
                },
                "hw_context": {
                    "devices": [{
                        "device_type": "device.cpu",
                        "description": "ARM SoC",
                        "attributes": {
                            "MHz": "600",
                            "Revision": "3",
                            "Implementer": "0x41"
                        }}, {
                        "device_type": "device.board",
                        "description": "Beagle Board C4",
                        "attributes": {
                            "Revision": "C4"
                        }
                    }]
                },
                "attributes": {
                    "testrun attr1": "value1",
                    "testrun attr2": "value2"
                },
                "attachments": {
                    "file.txt": [
                        "line 1\\n",
                        "line 2\\n"
                    ]
                }
            }
        ]
    }
    '''

    scenarios = [
        ('with_evolution', {
            "prefer_evolution": True
        }),
        ('without_evolution', {
            "prefer_evolution": False
        })
    ]

    def _attrs2set(self, attrs):
        """
        Convert a collection of Attribute model instances into a python
        frozenset of tuples (name, value).
        """
        return frozenset([(attr.name, attr.value) for attr in attrs.all()])

    def _pkgs2set(self, pkgs):
        """
        Convert a collection of SoftwarePackage model instances into a python
        frozenset of tuples (name, version).
        """
        return frozenset([(package.name, package.version) for package in pkgs])

    def _devs2set(self, devs):
        """
        Convert a collection of HardareDevice model instances into a python
        frozenset of tuples (device_type, description, attributes).
        """
        return frozenset([(
            device.device_type,
            device.description,
            self._attrs2set(device.attributes)
        ) for device in devs])

    def setUp(self):
        super(BundleDeserializerSuccessTests, self).setUp()
        self.s_bundle = fixtures.create_bundle(
            '/anonymous/', self.json_text, 'bundle.json')
        # Decompose the data here
        self.s_bundle.deserialize(prefer_evolution=self.prefer_evolution)
        if not self.s_bundle.is_deserialized:
            raise AssertionError("Deserialzation failed:" + self.s_bundle.deserialization_error.get().traceback)
        # Here we trick a little, since there is just one of each of
        # those models we can select them like this, the tests below
        # validate that we did not pick up some random object by
        # matching all the properties.
        self.s_test = Test.objects.all()[0]
        self.s_test_case = TestCaseModel.objects.all()[0]
        self.s_test_run = TestRun.objects.all()[0]
        self.s_test_result = TestResult.objects.all()[0]
        self.s_attachment = Attachment.objects.all()[0]

    def tearDown(self):
        self.s_bundle.delete_files()
        super(BundleDeserializerSuccessTests, self).tearDown()

    def test_Test__test_id(self):
        self.assertEqual(self.s_test.test_id, "some_test_id")

    def test_Test__name_is_empty(self):
        # Bundles have no way to convey this meta-data
        # Unless the test was named manually by operator
        # and existed prior to import it will not have a name
        self.assertEqual(self.s_test.name, "")

    def test_TestCase__test_is_same_as__Test(self):
        self.assertEqual(self.s_test_case.test, self.s_test)

    def test_TestCase__test_case_id(self):
        self.assertEqual(self.s_test_case.test_case_id, "some_test_case_id")

    def test_TestCase__name_is_empty(self):
        # Same as test_Test__name_is_empty above
        self.assertEqual(self.s_test_case.name, "")

    def test_TestCase__units(self):
        self.assertEqual(self.s_test_case.units, "bogomips")

    def test_TestRun__bundle(self):
        self.assertEqual(self.s_test_run.bundle, self.s_bundle)

    def test_TestRun__test(self):
        self.assertEqual(self.s_test_run.test, self.s_test)

    def test_TestRun__analyzer_assigned_uuid(self):
        self.assertEqual(
            self.s_test_run.analyzer_assigned_uuid,
            "1ab86b36-c23d-11df-a81b-002163936223")

    def test_TestRun__analyzer_assigned_date(self):
        self.assertEqual(
            self.s_test_run.analyzer_assigned_date,
            datetime.datetime(2010, 12, 31, 23, 59, 59, 0, None))

    def test_TestRun__time_check_performed(self):
        self.assertEqual(self.s_test_run.time_check_performed, True)

    def test_TestRun__sw_image_desc(self):
        self.assertEqual(self.s_test_run.sw_image_desc, "Ubuntu 10.10")

    def test_TestRun__packages(self):
        self.assertEqual(
            self._pkgs2set(self.s_test_run.packages.all()),
            frozenset([
                ("pkg1", "1.0"),
                ("pkg2", "0.5")]))

    def test_TestRun__devices(self):
        self.assertEqual(
            self._devs2set(self.s_test_run.devices.all()),
            frozenset([
                ("device.cpu", "ARM SoC", frozenset([
                    ("MHz", "600"),
                    ("Revision", "3"),
                    ("Implementer", "0x41")])
                ),
                ("device.board", "Beagle Board C4", frozenset([
                    ("Revision", "C4")])
                )]))

    def test_TestRun__attributes(self):
        self.assertEqual(
            self._attrs2set(self.s_test_run.attributes.all()),
            frozenset([
                ("testrun attr1", "value1"),
                ("testrun attr2", "value2")]))

    def test_TestRun__attachments(self):
        self.assertEqual(
            self.s_test_run.attachments.all()[0],
            self.s_attachment)

    def test_TestRun__attachment__content_filename(self):
        self.assertEqual(
            self.s_attachment.content_filename,
            "file.txt")

    def test_TestRun__attachment__content(self):
        self.assertEqual(
            self.s_attachment.content.read(),
            "line 1\nline 2\n")

    def test_TestResult__test_run(self):
        self.assertEqual(self.s_test_result.test_run, self.s_test_run)

    def test_TestResult__test_case(self):
        self.assertEqual(self.s_test_result.test_case, self.s_test_case)

    def test_TestResult__result(self):
        self.assertEqual(self.s_test_result.result, TestResult.RESULT_UNKNOWN)

    def test_TestResult__measurement(self):
        self.assertEqual(
            self.s_test_result.measurement,
            decimal.Decimal("1000.3"))

    def test_TestResult__units(self):
        self.assertEqual(self.s_test_result.units, "bogomips")

    def test_TestResult__filename(self):
        self.assertEqual(self.s_test_result.filename, "file.txt")

    def test_TestResult__lineno(self):
        self.assertEqual(self.s_test_result.lineno, 15)

    def test_TestResult__message(self):
        self.assertEqual(self.s_test_result.message, "text message")

    def test_TestResult__duration(self):
        self.assertEqual(
            self.s_test_result.duration,
            datetime.timedelta(days=1, seconds=1, microseconds=1))

    def test_TestResult__timestamp(self):
        self.assertEqual(
            self.s_test_result.timestamp,
            datetime.datetime(2010, 9, 17, 16, 34, 21, 0, None))

    def test_TestResult__attributes(self):
        self.assertEqual(
            self._attrs2set(self.s_test_result.attributes.all()),
            frozenset([
                ("attr1", "value1"),
                ("attr2", "value2")]))


class Bundle13DeserializerSuccessTests(TestCase):

    json_text = '''
    {
        "format": "Dashboard Bundle Format 1.3",
        "test_runs": [
            {
                "test_id": "some_test_id",
                "analyzer_assigned_uuid": "1ab86b36-c23d-11df-a81b-002163936223",
                "analyzer_assigned_date": "2010-12-31T23:59:59Z",
                "time_check_performed": true,
                "test_results": [ ],
                "tags": [
                    "tag-1",
                    "tag-2"
                ]
            }
        ]
    }
    '''
    
    def setUp(self):
        super(Bundle13DeserializerSuccessTests, self).setUp()
        self.s_bundle = fixtures.create_bundle(
            '/anonymous/', self.json_text, 'bundle.json')
        # Decompose the data here
        self.s_bundle.deserialize(prefer_evolution=False)
        if not self.s_bundle.is_deserialized:
            raise AssertionError("Deserialzation failed:" + self.s_bundle.deserialization_error.get().traceback)
        # Link to test run for easier testing
        self.s_test = self.s_bundle.test_runs.get()

    def tearDown(self):
        self.s_bundle.delete_files()
        super(Bundle13DeserializerSuccessTests, self).tearDown()

    def test_deserialize_tags(self):
        self.assertEqual(self.s_test.tags.count(), 2)
        self.assertEqual([tag.name for tag in self.s_test.tags.order_by('name').all()],
                         ["tag-1", "tag-2"])


class BundleDeserializerFailureTestCase(TestCaseWithScenarios):

    scenarios = [
        ("empty_string", {
            "json_text": '',
            "cause": ValueError,
        }),
        ("malformed_json", {
            "json_text": '{',
            "cause": ValueError,
        }),
        ("invalid_format", {
            "json_text": '{"format": "MS Excel with 50 sheets"}',
            "cause": DocumentFormatError
        }),
        ("invalid_datetime_value", {
            'json_text': """
            {
            "format": "Dashboard Bundle Format 1.0",
            "test_runs": [{
                    "test_id":  "some_test_id",
                    "test_results": [],
                    "time_check_performed": false,
                    "analyzer_assigned_uuid": "1ab86b36-c23d-11df-a81b-002163936223",
                    "analyzer_assigned_date": "9999-99-99T99:99:99Z"
                }]
            }
            """,
            "cause": ValidationError
        }),
        ("invalid_datetime_content", {
            'json_text': """
            {
            "format": "Dashboard Bundle Format 1.0",
            "test_runs": [{
                    "test_id":  "some_test_id",
                    "test_results": [],
                    "time_check_performed": false,
                    "analyzer_assigned_uuid": "1ab86b36-c23d-11df-a81b-002163936223",
                    "analyzer_assigned_date": {"nobody expected": "a dictionary"}
                }]
            }
            """,
            "cause": ValidationError
        }),
        ("invalid_uuid_value", {
            'json_text': """
            {
            "format": "Dashboard Bundle Format 1.0",
            "test_runs": [{
                    "test_id":  "some_test_id",
                    "test_results": [],
                    "time_check_performed": false,
                    "analyzer_assigned_uuid": "string that is not an uuid",
                    "analyzer_assigned_date": "2010-12-31T23:59:59Z"
                }]
            }
            """,
            "cause": ValueError
        }),
        ("invalid_uuid_content", {
            'json_text': """
            {
            "format": "Dashboard Bundle Format 1.0",
            "test_runs": [{
                    "test_id":  "some_test_id",
                    "test_results": [],
                    "time_check_performed": false,
                    "analyzer_assigned_uuid": 12345,
                    "analyzer_assigned_date": "2010-12-31T23:59:59Z"
                }]
            }
            """,
            "cause": ValidationError
        }),
        ("invalid_timedelta_content", {
            'json_text': """
            {
            "format": "Dashboard Bundle Format 1.0",
            "test_runs": [{
                    "test_id":  "some_test_id",
                    "test_results": [],
                    "time_check_performed": false,
                    "analyzer_assigned_uuid": "1ab86b36-c23d-11df-a81b-002163936223",
                    "analyzer_assigned_date": "2010-12-31T23:59:59Z",
                    "test_results": [{
                        "result": "pass",
                        "duration": 19123123123123123132
                    }]
                }]
            }
            """,
            "cause": ValidationError
        }),
    ]

    def setUp(self):
        super(BundleDeserializerFailureTestCase, self).setUp()
        self.s_bundle = fixtures.create_bundle(
            '/anonymous/', self.json_text, 'bundle.json')

    def tearDown(self):
        self.s_bundle.delete_files()
        super(BundleDeserializerFailureTestCase, self).tearDown()

    def test_deserializer_failure_without_evolution(self):
        try:
            BundleDeserializer().deserialize(self.s_bundle, prefer_evolution=False)
        except Exception as ex:
            self.assertIsInstance(ex, self.cause)
        else:
            self.fail("Should have raised an exception")

    def test_deserializer_failure_with_evolution(self):
        try:
            BundleDeserializer().deserialize(self.s_bundle, prefer_evolution=True)
        except Exception as ex:
            self.assertIsInstance(ex, self.cause)
        else:
            self.fail("Should have raised an exception")


class BundleDeserializerAtomicityTestCase(TransactionTestCase):
    # Importing this bundle will fail as analyzer_assigned_uuid is not
    # unique. Due to proper transaction handling the first test run
    # model instance will not be visible after the failed upload
    json_text = """
    {
        "format": "Dashboard Bundle Format 1.0",
        "test_runs": [
            {
                "test_id": "some_test_id",
                "analyzer_assigned_uuid": "1ab86b36-c23d-11df-a81b-002163936223",
                "analyzer_assigned_date": "2010-12-31T23:59:59Z",
                "time_check_performed": true,
                "test_results": []
            }, {
                "test_id": "some_test_id",
                "analyzer_assigned_uuid": "1ab86b36-c23d-11df-a81b-002163936223",
                "analyzer_assigned_date": "2010-12-31T23:59:59Z",
                "time_check_performed": true,
                "test_results": []
            }
        ]
    }
    """

    def setUp(self):
        super(BundleDeserializerAtomicityTestCase, self).setUp()
        self.s_bundle = fixtures.create_bundle(
            '/anonymous/', self.json_text, 'bundle.json')

    def tearDown(self):
        self.s_bundle.delete_files()
        super(BundleDeserializerAtomicityTestCase, self).tearDown()

    def test_bundle_deserialization_failed(self):
        self.s_bundle.deserialize()
        self.assertFalse(self.s_bundle.is_deserialized)

    def test_bundle_count(self):
        self.s_bundle.deserialize()
        self.assertEqual(Bundle.objects.count(), 1)

    def test_bundle_deserialization_error_count(self):
        self.s_bundle.deserialize()
        self.assertEqual(BundleDeserializationError.objects.count(), 1)

    def test_error_trace(self):
        self.s_bundle.deserialize()
        # The message depends on the database. This is a little ugly but it's
        # better than not knowing what really happened and hiding other
        # potential bugs that would otherwise be masked here.
        self.assertIn(
            self.s_bundle.deserialization_error.error_message, [
                'A test with UUID 1ab86b36-c23d-11df-a81b-002163936223 already exists',
                'column analyzer_assigned_uuid is not unique',
                u'duplicate key value violates unique constraint '
                u'"dashboard_app_testrun_analyzer_assigned_uuid_key"\n'])

    def test_deserialization_failure_does_not_leave_junk_behind(self):
        self.s_bundle.deserialize()
        self.assertRaises(
            TestRun.DoesNotExist, TestRun.objects.get,
            analyzer_assigned_uuid="1ab86b36-c23d-11df-a81b-002163936223")



"""
Tests for the Bundle model
"""
import hashlib

from django.core.files.base import ContentFile
from django.test import TestCase
from django_testscenarios.ubertest import TestCaseWithScenarios
from mocker import Mocker, expect

from dashboard_app.tests import fixtures
from dashboard_app.models import (
    Bundle,
    BundleDeserializationError,
    BundleStream,
)
from dashboard_app.tests.call_helper import ObjectFactoryMixIn


class BundleTests(TestCase, ObjectFactoryMixIn):

    class Dummy:
        class Bundle:
            @property
            def bundle_stream(self):
                from django.contrib.auth.models import User
                user = User.objects.get_or_create(username="dummy_user")[0]
                return BundleStream.objects.get_or_create(slug="foobar", user=user)[0]
            uploaded_by = None
            content = ContentFile("file content")
            content_filename = "file.txt"

    def test_construction(self):
        dummy, bundle = self.make_and_get_dummy(Bundle)
        bundle.content.save(bundle.content_filename, dummy.content)
        # reset the dummy content file pointer for subsequent tests
        dummy.content.seek(0)
        content = dummy.content.read()

        bundle.save()
        try:
            self.assertEqual(bundle.bundle_stream, dummy.bundle_stream)
            self.assertEqual(bundle.uploaded_by, dummy.uploaded_by)
            #self.assertEqual(bundle.uploaded_on, mocked_value_of_time.now)
            self.assertEqual(bundle.is_deserialized, False)
            bundle.content.open()
            self.assertEqual(bundle.content.read(), content)
            bundle.content.close()
            self.assertEqual(bundle.content_sha1,
                    hashlib.sha1(content).hexdigest())
            self.assertEqual(bundle.content_filename,
                    dummy.content_filename)
        finally:
            bundle.delete_files()

    def test_unicode(self):
        obj = Bundle(content_filename="file.json", pk=1)
        self.assertEqual(unicode(obj), u"Bundle None")


class BundleDeserializationTests(TestCaseWithScenarios):

    scenarios = [
        ('dummy_import_failure', {
            'pathname': '/public/personal/admin/',
            'is_public': 'true',
            'content': 'bogus',
            'content_filename': 'test1.json',
        }),
    ]

    def setUp(self):
        super(BundleDeserializationTests, self).setUp()
        self.bundle = fixtures.create_bundle(
            self.pathname, self.content, self.content_filename)
        self.mocker = Mocker()

    def tearDown(self):
        self.bundle.delete_files()
        self.mocker.restore()
        self.mocker.verify()
        super(BundleDeserializationTests, self).tearDown()

    def test_deserialize_failure_leaves_trace(self):
        mock = self.mocker.patch(self.bundle)
        expect(mock._do_deserialize(False)).throw(Exception("boom"))
        self.mocker.replay()
        self.bundle.deserialize(False)
        self.assertFalse(self.bundle.is_deserialized)
        self.assertEqual(self.bundle.deserialization_error.error_message, "boom")

    def test_deserialize_ignores_deserialized_bundles(self):
        # just reply as we're not using mocker in this test case 
        self.mocker.replay()
        self.bundle.is_deserialized = True
        self.bundle.deserialize(False)
        self.assertTrue(self.bundle.is_deserialized)

    def test_deserialize_sets_is_serialized_on_success(self):
        mock = self.mocker.patch(self.bundle)
        expect(mock._do_deserialize(False))
        self.mocker.replay()
        self.bundle.deserialize(False)
        self.assertTrue(self.bundle.is_deserialized)

    def test_deserialize_clears_old_error_on_success(self):
        BundleDeserializationError.objects.create(
            bundle = self.bundle,
            error_message="not important").save()
        mock = self.mocker.patch(self.bundle)
        expect(mock._do_deserialize(False))
        self.mocker.replay()
        self.bundle.deserialize(False)
        # note we cannot check for self.bundle.deserialization_error
        # directly due to the way django handles operations that affect
        # existing instances (it does not touch them like storm would
        # IIRC).
        self.assertRaises(
            BundleDeserializationError.DoesNotExist,
            BundleDeserializationError.objects.get, bundle=self.bundle)



# Copyright (C) 2010 Linaro Limited
#
# Author: Guilherme Salgado <guilherme.salgado@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Tests for the login bits of Launch Control.
"""
import cgi
import httplib

from django.conf import settings
from django.contrib.auth.models import User
from django.test import TestCase

from django_openid_auth.models import UserOpenID
from django_openid_auth.tests.test_views import StubOpenIDProvider

from openid.fetchers import setDefaultFetcher

from dashboard_app.tests.utils import TestClient


class TestOpenIDLogin(TestCase):

    urls = 'django_openid_auth.tests.urls'
    _username = 'someuser'
    _identity_url = 'http://example.com/identity'

    def test_positive_response_from_provider(self):
        self.create_user()
        openid_request = self.initiate_login()

        # Simulate a positive assertion from the server.
        openid_response = openid_request.answer(True)

        # Use that response to complete the authentication.
        response = self.complete(openid_response)

        # And the user is now logged in.
        response = self.client.get('/getuser/')
        self.assertEquals(response.content, self._username)

    def test_negative_response_from_provider(self):
        openid_request = self.initiate_login()

        # Simulate a negative assertion from the server.
        openid_response = openid_request.answer(False)

        # Use that response to complete the authentication.
        response = self.complete(openid_response)

        # Since we got a negative assertion from the server, no user is logged
        # in.
        response = self.client.get('/getuser/')
        self.assertEquals(response.content, '')

    def setUp(self):
        super(TestOpenIDLogin, self).setUp()
        # Use StubOpenIDProvider and _identity_url as our fixed SSO so that
        # we always get a successful OpenID response for _identity_url.
        self.provider = StubOpenIDProvider('http://example.com/')
        setDefaultFetcher(self.provider, wrap_exceptions=False)
        self.missing_sso_server_url = object()
        self.orig_sso_server_url = getattr(
            settings, 'OPENID_SSO_SERVER_URL', self.missing_sso_server_url)
        settings.OPENID_SSO_SERVER_URL = self._identity_url
        self.client = TestClient()

    def tearDown(self):
        super(TestOpenIDLogin, self).tearDown()
        setDefaultFetcher(None)
        if self.orig_sso_server_url == self.missing_sso_server_url:
            del settings.OPENID_SSO_SERVER_URL
        else:
            settings.OPENID_SSO_SERVER_URL = self.orig_sso_server_url

    def create_user(self):
        user = User(username=self._username)
        user.save()
        # Associate our newly created user with the identity URL.
        useropenid = UserOpenID(
            user=user, claimed_id=self._identity_url,
            display_id=self._identity_url)
        useropenid.save()

    def initiate_login(self):
        response = self.client.get('/openid/login/')
        self.assertEqual(httplib.OK, response.status_code)
        openid_request = self.provider.parseFormPost(response.content)
        self.assertTrue(openid_request.return_to.startswith(
            'http://testserver/openid/complete/'))
        return openid_request

    def complete(self, openid_response):
        """Complete an OpenID authentication request."""
        webresponse = self.provider.server.encodeResponse(openid_response)
        self.assertEquals(webresponse.code, 302)
        redirect_to = webresponse.headers['location']
        self.assertTrue(redirect_to.startswith(
            'http://testserver/openid/complete/'))
        return self.client.get('/openid/complete/',
            dict(cgi.parse_qsl(redirect_to.split('?', 1)[1])))



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):

    def forwards(self, orm):
        # Result codes
        RESULT_PASS = 0
        RESULT_FAIL = 1
        RESULT_SKIP = 2
        RESULT_UNKNOWN = 3
        # Code-to-name mapping
        RESULT_MAP = {
            RESULT_PASS: 'pass',
            RESULT_FAIL: 'fail',
            RESULT_SKIP: 'skip',
            RESULT_UNKNOWN: 'unknown'
        }
        for test_run in orm.TestRun.objects.all():
            stats = test_run.test_results.order_by(
                # Disable sorting
            ).values(
                'result'  # only get the result outcome (pass/fail/etc)
            ).annotate(
                count=models.Count('result')  # Count number of items with this value
            )
            # Translate the 0-4 element array into a 0-4 element dictionary 
            result = dict([
                (RESULT_MAP[item['result']], item['count'])
                for item in stats])
            # Create a denormalized test run instance
            orm.TestRunDenormalization.objects.create(
                test_run=test_run,
                count_pass=result.get('pass', 0),
                count_fail=result.get('fail', 0),
                count_skip=result.get('skip', 0),
                count_unknown=result.get('unknown', 0))

    def backwards(self, orm):
        orm.TestRunDenormalization.objects.all().delete()

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'dashboard_app.attachment': {
            'Meta': {'object_name': 'Attachment'},
            'content': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True'}),
            'content_filename': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'mime_type': ('django.db.models.fields.CharField', [], {'max_length': '64'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'public_url': ('django.db.models.fields.URLField', [], {'max_length': '512', 'blank': 'True'})
        },
        'dashboard_app.bundle': {
            'Meta': {'ordering': "['-uploaded_on']", 'object_name': 'Bundle'},
            'bundle_stream': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'bundles'", 'to': "orm['dashboard_app.BundleStream']"}),
            'content': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True'}),
            'content_filename': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'content_sha1': ('django.db.models.fields.CharField', [], {'max_length': '40', 'unique': 'True', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_deserialized': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'uploaded_by': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'uploaded_bundles'", 'null': 'True', 'to': "orm['auth.User']"}),
            'uploaded_on': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.utcnow'})
        },
        'dashboard_app.bundledeserializationerror': {
            'Meta': {'object_name': 'BundleDeserializationError'},
            'bundle': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'deserialization_error'", 'unique': 'True', 'primary_key': 'True', 'to': "orm['dashboard_app.Bundle']"}),
            'error_message': ('django.db.models.fields.CharField', [], {'max_length': '1024'}),
            'traceback': ('django.db.models.fields.TextField', [], {'max_length': '32768'})
        },
        'dashboard_app.bundlestream': {
            'Meta': {'object_name': 'BundleStream'},
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.Group']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_anonymous': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'pathname': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '128'}),
            'slug': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'dashboard_app.hardwaredevice': {
            'Meta': {'object_name': 'HardwareDevice'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'device_type': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        'dashboard_app.namedattribute': {
            'Meta': {'unique_together': "(('object_id', 'name'),)", 'object_name': 'NamedAttribute'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '512'})
        },
        'dashboard_app.softwarepackage': {
            'Meta': {'unique_together': "(('name', 'version'),)", 'object_name': 'SoftwarePackage'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'dashboard_app.softwarepackagescratch': {
            'Meta': {'object_name': 'SoftwarePackageScratch'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'version': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'dashboard_app.softwaresource': {
            'Meta': {'object_name': 'SoftwareSource'},
            'branch_revision': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'branch_url': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'branch_vcs': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'commit_timestamp': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'project_name': ('django.db.models.fields.CharField', [], {'max_length': '32'})
        },
        'dashboard_app.tag': {
            'Meta': {'object_name': 'Tag'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '256', 'db_index': 'True'})
        },
        'dashboard_app.test': {
            'Meta': {'object_name': 'Test'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'test_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '64'})
        },
        'dashboard_app.testcase': {
            'Meta': {'unique_together': "(('test', 'test_case_id'),)", 'object_name': 'TestCase'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'test': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'test_cases'", 'to': "orm['dashboard_app.Test']"}),
            'test_case_id': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'units': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'})
        },
        'dashboard_app.testingeffort': {
            'Meta': {'object_name': 'TestingEffort'},
            'description': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'project': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testing_efforts'", 'to': "orm['lava_projects.Project']"}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'related_name': "'testing_efforts'", 'symmetrical': 'False', 'to': "orm['dashboard_app.Tag']"})
        },
        'dashboard_app.testresult': {
            'Meta': {'ordering': "('_order',)", 'object_name': 'TestResult'},
            '_order': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'filename': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lineno': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'measurement': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '20', 'decimal_places': '10', 'blank': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            'microseconds': ('django.db.models.fields.BigIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'relative_index': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'result': ('django.db.models.fields.PositiveSmallIntegerField', [], {}),
            'test_case': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'test_results'", 'null': 'True', 'to': "orm['dashboard_app.TestCase']"}),
            'test_run': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'test_results'", 'to': "orm['dashboard_app.TestRun']"}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'dashboard_app.testrun': {
            'Meta': {'ordering': "['-import_assigned_date']", 'object_name': 'TestRun'},
            'analyzer_assigned_date': ('django.db.models.fields.DateTimeField', [], {}),
            'analyzer_assigned_uuid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '36'}),
            'bundle': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'test_runs'", 'to': "orm['dashboard_app.Bundle']"}),
            'devices': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'test_runs'", 'blank': 'True', 'to': "orm['dashboard_app.HardwareDevice']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'import_assigned_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'packages': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'test_runs'", 'blank': 'True', 'to': "orm['dashboard_app.SoftwarePackage']"}),
            'sources': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'test_runs'", 'blank': 'True', 'to': "orm['dashboard_app.SoftwareSource']"}),
            'sw_image_desc': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'tags': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'test_runs'", 'blank': 'True', 'to': "orm['dashboard_app.Tag']"}),
            'test': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'test_runs'", 'to': "orm['dashboard_app.Test']"}),
            'time_check_performed': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'dashboard_app.testrundenormalization': {
            'Meta': {'object_name': 'TestRunDenormalization'},
            'count_fail': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'count_pass': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'count_skip': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'count_unknown': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'test_run': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'denormalization'", 'unique': 'True', 'primary_key': 'True', 'to': "orm['dashboard_app.TestRun']"})
        },
        'lava_projects.project': {
            'Meta': {'object_name': 'Project'},
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.Group']", 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identifier': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'is_aggregate': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'registered_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'projects'", 'to': "orm['auth.User']"}),
            'registered_on': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['dashboard_app']



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

from mocker import Mocker, expect
from testtools import TestCase

from dashboard_app.models import DataView


class DataViewHandlerTests(TestCase):

    text = """
    <data-view name="foo">
        <documentation>
        Simple dataview that selects all bundle streams
        </documentation>
        <summary>List all bundle streams</summary>
        <sql backend="sqlite">
            SELECT *
            FROM dashboard_app_bundlestreams
            ORDER BY <value name="order_by"/>
        </sql>
        <arguments>
            <argument name="order_by" default="pathname" type="string" help="sorting order"/>
        </arguments>
    </data-view>
    """

    def setUp(self):
        super(DataViewHandlerTests, self).setUp()
        self.dataview = DataView.repository.load_from_xml_string(self.text) 

    def test_name_parsed_ok(self):
        self.assertEqual(self.dataview.name, "foo")

    def test_documentation_parsed_ok(self):
        self.assertEqual(
            self.dataview.documentation,
            "Simple dataview that selects all bundle streams")

    def test_summary_parsed_ok(self):
        self.assertEqual(self.dataview.summary, "List all bundle streams")

    def test_sql_parsed_ok(self):
        self.assertTrue("sqlite" in self.dataview.backend_queries)
        backend_query = self.dataview.backend_queries["sqlite"]
        self.assertEqual(backend_query.backend, "sqlite")
        self.assertEqual(
            backend_query.sql_template,
            "SELECT * FROM dashboard_app_bundlestreams ORDER BY {order_by}")
        self.assertEqual(backend_query.argument_list, ["order_by"])

    def test_arguments_parsed_ok(self):
        self.assertEqual(len(self.dataview.arguments), 1)
        arg = self.dataview.arguments[0]
        self.assertEqual(arg.name, "order_by")
        self.assertEqual(arg.default, "pathname")
        self.assertEqual(arg.type, "string")
        self.assertEqual(arg.help, "sorting order")


class DataViewConnectionTests(TestCase):

    def test_get_connection_without_sandbox(self):
        # Mock connections['dataview'] to raise ConnectionDoesNotExist
        from django.db.utils import ConnectionDoesNotExist
        mocker = Mocker()
        connections = mocker.replace("django.db.connections")
        expect(connections['dataview']).throw(ConnectionDoesNotExist)
        connection = mocker.replace("django.db.connection")
        logging = mocker.replace("logging")
        logging.warning("dataview-specific database connection not available, dataview query is NOT sandboxed")
        with mocker:
            observed = DataView.get_connection()
            expected = connection
            self.assertIs(observed, expected)

    def test_get_connection_with_sandbox(self):
        """
        Test for DataView.get_connection()
        """
        # Mock connections['dataview'] to return special connection 
        mocker = Mocker()
        connections = mocker.replace("django.db.connections")
        special_connection = mocker.mock()
        expect(connections['dataview']).result(special_connection)
        with mocker:
            observed = DataView.get_connection()
            expected = special_connection
            self.assertIs(observed, expected)



"""
Test for the SoftwarePackage model
"""

from django.db import IntegrityError
from django.test import TestCase

from dashboard_app.models import SoftwarePackage
from dashboard_app.tests.call_helper import ObjectFactoryMixIn


class SoftwarePackageTestCase(TestCase, ObjectFactoryMixIn):

    class Dummy:
        class SoftwarePackage:
            name = 'libfoo'
            version = '1.2.0'

    def test_creation_1(self):
        dummy, sw_package = self.make_and_get_dummy(SoftwarePackage)
        sw_package.save()
        self.assertEqual(sw_package.name, dummy.name)
        self.assertEqual(sw_package.version, dummy.version)

    def test_LP744922(self):
        """
        Regression test for https://bugs.launchpad.net/launch-control/+bug/744922
        """
        sw_package = SoftwarePackage.objects.create(name='foo', version='x' * 33)
        sw_package.save()
        self.assertEqual(len(sw_package.version), 33)

    def test_uniqueness(self):
        pkg1 = self.make(SoftwarePackage)
        pkg1.save()
        pkg2 = self.make(SoftwarePackage)
        self.assertRaises(IntegrityError, pkg2.save)

    def test_unicode(self):
        obj = SoftwarePackage(name="foo", version="1.2")
        self.assertEqual(unicode(obj), u"foo 1.2")



"""
Module with non-database helper classes
"""

from uuid import UUID
import base64
import logging
import time

from django.core.files.base import ContentFile
from django.db import connection, transaction, IntegrityError
from linaro_dashboard_bundle.errors import DocumentFormatError
from linaro_dashboard_bundle.evolution import DocumentEvolution
from linaro_dashboard_bundle.io import DocumentIO
from linaro_json.extensions import datetime_extension, timedelta_extension


PROFILE_LOGGING = False


def is_postgres():
    return (connection.settings_dict['ENGINE']
            == 'django.db.backends.postgresql_psycopg2')


class IBundleFormatImporter(object):
    """
    Interface for bundle format importers.
    """

    def import_document(self, s_bundle, doc):
        """
        Import document in a supported format and tie it to the
        specified server side DashboardBundle model (s_bundle)

        :Discussion:
            Imports specified document in a supported format and tie it
            to the specified server side DashboardBundle model
            (s_bundle)

        :Return value:
            None

        :Exceptions raised:
            ValueError
            TypeError
            others (?)
        """
        raise NotImplementedError(self.import_document)


class BundleFormatImporter_1_0(IBundleFormatImporter):
    """
    IFormatImporter subclass capable of loading "Dashboard Bundle Format 1.0"
    """

    def import_document(self, s_bundle, doc):
        """
        Import specified bundle document into the database.
        """
        self._content_files = []
        self._import_sanity_check(doc)
        try:
            self._import_document_with_transaction(s_bundle, doc)
        except IntegrityError:
            self._remove_created_files()
            raise

    def _remove_created_files(self):
        """
        Remove any files that may have already been flushed to disk. Otherwise
        the transaction handling should rollback anything evil that was
        happening while we were here
        """
        for content_file in self._content_files:
            content_file.delete(save=False)

    def _import_sanity_check(self, doc):
        """
        Sanity check before any import is attempted.

        This prevents InternalError (but is racy with other transactions).
        Still it's a little bit better to report the exception raised below
        rather than the IntegrityError that would have been raised otherwise.

        The code copes with both (using transactions around _import_document()
        and _remove_created_files() that gets called if something is wrong)
        """
        from dashboard_app.models import TestRun

        for test_run in doc.get("test_runs", []):
            analyzer_assigned_uuid = test_run["analyzer_assigned_uuid"]
            if TestRun.objects.filter(
                analyzer_assigned_uuid=analyzer_assigned_uuid).exists():
                raise ValueError("A test with UUID {0} already exists".format(analyzer_assigned_uuid))

    @transaction.commit_on_success
    def _import_document_with_transaction(self, s_bundle, doc):
        """
        Note: This function uses commit_on_success to ensure the database is in
        a consistent state after IntegrityErrors that would clog the
        transaction on pgsql. Since transactions will not rollback any files we
        created in the meantime there is a helper that cleans attachments in
        case something goes wrong
        """
        self._import_document(s_bundle, doc)

    def _import_document(self, s_bundle, doc):
        for c_test_run in doc.get("test_runs", []):
            self._import_test_run(c_test_run, s_bundle)

    def __init__(self):
        self._qc = 0
        self._time = time.time()

    def _log(self, method_name):
        if PROFILE_LOGGING:
            t = time.time() - self._time
            if t > 0.1:
                p = '**'
            else:
                p = '  '
            logging.warning(
                '%s %s %.2f %d', p, method_name, t,
                len(connection.queries) - self._qc)
            self._qc = len(connection.queries)
            self._time = time.time()

    def _import_test_run(self, c_test_run, s_bundle):
        """
        Import TestRun
        """
        from dashboard_app.models import TestRun

        s_test = self._import_test(c_test_run)
        analyzer_assigned_uuid = UUID(c_test_run["analyzer_assigned_uuid"])
        s_test_run = TestRun.objects.create(
            bundle = s_bundle,
            test = s_test,
            analyzer_assigned_uuid = str(analyzer_assigned_uuid),
            analyzer_assigned_date = datetime_extension.from_json(
                # required by schema
                c_test_run["analyzer_assigned_date"]),
            time_check_performed = (
                # required by schema
                c_test_run["time_check_performed"]),
        )
        # needed for foreign key models below
        s_test_run.save()
        # import all the bits and pieces
        self._log('starting')
        self._import_test_results(c_test_run, s_test_run)
        self._log('results')
        self._import_attachments(c_test_run, s_test_run)
        self._log('attachments')
        self._import_hardware_context(c_test_run, s_test_run)
        self._log('hardware')
        self._import_software_context(c_test_run, s_test_run)
        self._log('software')
        self._import_attributes(c_test_run, s_test_run)
        self._log('attributes')
        # collect all the changes that happen before the previous save
        s_test_run.save()
        s_test_run.denormalize()
        return s_test_run

    def _import_software_context(self, c_test_run, s_test_run):
        """
        Import software context.

        In format 1.0 that's just a list of packages and software image
        description
        """
        self._import_packages(c_test_run, s_test_run)
        s_test_run.sw_image_desc = self._get_sw_context(c_test_run).get(
                "sw_image", {}).get("desc", "")

    def _import_hardware_context(self, c_test_run, s_test_run):
        """
        Import hardware context.

        In format 1.0 that's just a list of devices
        """
        self._import_devices(c_test_run, s_test_run)

    def _import_test(self, c_test_run):
        """
        Import dashboard_app.models.Test into the database
        based on a client-side description of a TestRun
        """
        from dashboard_app.models import Test

        s_test, test_created = Test.objects.get_or_create(
            test_id = c_test_run["test_id"]) # required by schema
        if test_created:
            s_test.save()
        return s_test

    def _import_test_results_sqlite(self, c_test_results, s_test_run):
        cursor = connection.cursor()

        # XXX I don't understand how the _order column that Django adds is
        # supposed to work.  I just set it to 0 here.

        data = []

        for index, c_test_result in enumerate(c_test_results, 1):

            timestamp = c_test_result.get("timestamp")
            if timestamp:
                timestamp = datetime_extension.from_json(timestamp)
            duration = c_test_result.get("duration", None)
            if duration:
                duration = timedelta_extension.from_json(duration)
                duration = (duration.microseconds +
                            (duration.seconds * 10 ** 6) +
                            (duration.days * 24 * 60 * 60 * 10 ** 6))
            result = self._translate_result_string(c_test_result["result"])

            data.append((
                s_test_run.id,
                index,
                timestamp,
                duration,
                c_test_result.get("log_filename", None),
                result,
                c_test_result.get("measurement", None),
                c_test_result.get("message", None),
                c_test_result.get("log_lineno", None),
                s_test_run.test.id,
                c_test_result.get("test_case_id", None),
                ))

        cursor.executemany(
            """
            INSERT INTO dashboard_app_testresult (
                test_run_id,
                relative_index,
                timestamp,
                microseconds,
                filename,
                result,
                measurement,
                message,
                lineno,
                _order,
                test_case_id
            ) select
                %s,
                %s,
                %s,
                %s,
                %s,
                %s,
                %s,
                %s,
                %s,
                0,
                dashboard_app_testcase.id
                FROM dashboard_app_testcase
                  WHERE dashboard_app_testcase.test_id = %s
                    AND dashboard_app_testcase.test_case_id
                      = %s
            """, data)

        cursor.close()

    def _import_test_results_pgsql(self, c_test_results, s_test_run):
        cursor = connection.cursor()

        # XXX I don't understand how the _order column that Django adds is
        # supposed to work.  I just let it default to 0 here.

        data = []

        for i in range(0, len(c_test_results), 1000):

            cursor.execute(
                """
                CREATE TEMPORARY TABLE newtestresults (
                    relative_index INTEGER,
                    timestamp      TIMESTAMP WITH TIME ZONE,
                    microseconds   BIGINT,
                    filename       TEXT,
                    result         SMALLINT,
                    measurement    NUMERIC(20,10),
                    message        TEXT,
                    test_case_id   TEXT,
                    lineno         INTEGER
                    )
                """)

            data = []

            for index, c_test_result in enumerate(c_test_results[i:i+1000], i+1):

                timestamp = c_test_result.get("timestamp")
                if timestamp:
                    timestamp = datetime_extension.from_json(timestamp)
                duration = c_test_result.get("duration", None)
                if duration:
                    duration = timedelta_extension.from_json(duration)
                    duration = (duration.microseconds +
                                (duration.seconds * 10 ** 6) +
                                (duration.days * 24 * 60 * 60 * 10 ** 6))
                result = self._translate_result_string(c_test_result["result"])

                data.extend([
                    index,
                    timestamp,
                    duration,
                    c_test_result.get("log_filename", None),
                    result,
                    c_test_result.get("measurement", None),
                    c_test_result.get("message", None),
                    c_test_result.get("test_case_id", None),
                    c_test_result.get("log_lineno", None),
                    ])

            sequel = ',\n'.join(
                ["(" + "%s" % (', '.join(['%s']*9),) + ")"] * (len(data) // 9))

            cursor.execute(
                """
                INSERT INTO newtestresults (
                    relative_index,
                    timestamp,
                    microseconds,
                    filename,
                    result,
                    measurement,
                    message,
                    test_case_id,
                    lineno
                ) VALUES """ + sequel, data)

            cursor.execute(
                """
                INSERT INTO dashboard_app_testresult (
                    test_run_id,
                    relative_index,
                    timestamp,
                    microseconds,
                    filename,
                    result,
                    measurement,
                    message,
                    test_case_id,
                    lineno
                ) SELECT
                    %s,
                    relative_index,
                    timestamp,
                    microseconds,
                    filename,
                    result,
                    measurement,
                    message,
                    dashboard_app_testcase.id,
                    lineno
                    FROM newtestresults, dashboard_app_testcase
                      WHERE dashboard_app_testcase.test_id = %s
                        AND dashboard_app_testcase.test_case_id
                          = newtestresults.test_case_id
                """ % (s_test_run.id, s_test_run.test.id))

            cursor.execute(
                """
                DROP TABLE newtestresults
                """)

        cursor.close()

    def _import_test_results(self, c_test_run, s_test_run):
        """
        Import TestRun.test_results
        """
        from dashboard_app.models import TestResult

        c_test_results = c_test_run.get("test_results", [])

        if not c_test_results:
            return

        if is_postgres():
            self._import_test_cases_pgsql(c_test_results, s_test_run.test)
        else:
            self._import_test_cases_sqlite(c_test_results, s_test_run.test)

        if is_postgres():
            self._import_test_results_pgsql(c_test_results, s_test_run)
        else:
            self._import_test_results_sqlite(c_test_results, s_test_run)

        for index, c_test_result in enumerate(c_test_run.get("test_results", []), 1):
            if c_test_result.get("attributes", {}):
                s_test_result = TestResult.objects.get(
                    relative_index=index, test_run=s_test_run)
                self._import_attributes(c_test_result, s_test_result)
        self._log('test result attributes')

    def _import_test_cases_sqlite(self, c_test_results, s_test):
        """
        Import TestCase
        """
        id_units = []
        for c_test_result in c_test_results:
            if "test_case_id" not in c_test_result:
                continue
            id_units.append(
                (c_test_result["test_case_id"],
                 c_test_result.get("units", "")))

        cursor = connection.cursor()

        data = []
        for (test_case_id, units) in id_units:
            data.append((s_test.id, units, test_case_id))
        cursor.executemany(
            """
            INSERT OR IGNORE INTO
                dashboard_app_testcase (test_id, units, name, test_case_id)
            VALUES (%s, %s, '', %s)
            """, data)
        cursor.close()

    def _import_test_cases_pgsql(self, c_test_results, s_test):
        """
        Import TestCase
        """
        id_units = {}
        for c_test_result in c_test_results:
            if "test_case_id" not in c_test_result:
                continue
            id_units.setdefault(
                c_test_result["test_case_id"], c_test_result.get("units", ""))
        id_units = id_units.items()

        cursor = connection.cursor()
        for i in range(0, len(id_units), 1000):

            cursor.execute(
                """
                CREATE TEMPORARY TABLE
                    newtestcases (test_case_id text, units text)
                """)
            data = []
            for (id, units) in id_units[i:i+1000]:
                data.append(id)
                data.append(units)
            sequel = ',\n'.join(["(%s, %s)"] * (len(data) // 2))
            cursor.execute(
                """
                INSERT INTO newtestcases (test_case_id, units) VALUES
                """ + sequel, data)

            cursor.execute(
                """
                INSERT INTO
                    dashboard_app_testcase (test_id, units, name, test_case_id)
                SELECT %s, units, E'', test_case_id FROM newtestcases
                WHERE NOT EXISTS (SELECT 1 FROM dashboard_app_testcase
                                  WHERE test_id = %s
                                    AND newtestcases.test_case_id
                                      = dashboard_app_testcase.test_case_id)
                """ % (s_test.id, s_test.id))
            cursor.execute(
                """
                drop table newtestcases
                """)
        cursor.close()

    def _import_packages_scratch_sqlite(self, cursor, packages):
        data = []
        for c_package in packages:
            data.append((c_package['name'], c_package['version']))
        cursor.executemany(
            """
            INSERT INTO dashboard_app_softwarepackagescratch
                   (name, version) VALUES (%s, %s)
            """, data)

    def _import_packages_scratch_pgsql(self, cursor, packages):
        for i in range(0, len(packages), 1000):
            data = []
            for c_package in packages[i:i+1000]:
                data.append(c_package['name'])
                data.append(c_package['version'])
            sequel = ', '.join(["(%s, %s)"] * (len(data) // 2))
            cursor.execute(
                "INSERT INTO dashboard_app_softwarepackagescratch (name, version) VALUES " + sequel, data)

    def _import_packages(self, c_test_run, s_test_run):
        """
        Import TestRun.pacakges
        """
        packages = self._get_sw_context(c_test_run).get("packages", [])
        if not packages:
            return
        cursor = connection.cursor()

        if is_postgres():
            self._import_packages_scratch_pgsql(cursor, packages)
        else:
            self._import_packages_scratch_sqlite(cursor, packages)

        cursor.execute(
            """
            INSERT INTO dashboard_app_softwarepackage (name, version)
            SELECT name, version FROM dashboard_app_softwarepackagescratch
            EXCEPT SELECT name, version FROM dashboard_app_softwarepackage
            """)
        cursor.execute(
            """
            INSERT INTO
                dashboard_app_testrun_packages (testrun_id, softwarepackage_id)
            SELECT %s, id FROM dashboard_app_softwarepackage
                WHERE EXISTS (
                    SELECT * FROM dashboard_app_softwarepackagescratch
                        WHERE dashboard_app_softwarepackage.name
                            = dashboard_app_softwarepackagescratch.name
                          AND dashboard_app_softwarepackage.version
                            = dashboard_app_softwarepackagescratch.version)
            """ % s_test_run.id)
        cursor.execute(
            """
            delete from dashboard_app_softwarepackagescratch
            """)
        cursor.close()

    def _import_devices(self, c_test_run, s_test_run):
        """
        Import TestRun.devices
        """
        from dashboard_app.models import HardwareDevice

        for c_device in self._get_hw_context(c_test_run).get("devices", []):
            s_device = HardwareDevice.objects.create(
                device_type = c_device["device_type"],
                description = c_device["description"]
            )
            s_device.save()
            self._import_attributes(c_device, s_device)
            s_test_run.devices.add(s_device)

    def _import_attributes(self, c_object, s_object):
        """
        Import attributes from any client-side object into any
        server-side object
        """
        for name, value in c_object.get("attributes", {}).iteritems():
            s_object.attributes.create(
                name=str(name), value=str(value))

    def _import_attachments(self, c_test_run, s_test_run):
        """
        Import TestRun.attachments
        """
        for filename, lines in c_test_run.get("attachments", {}).iteritems():
            s_attachment = s_test_run.attachments.create(
                mime_type="text/plain",
                content_filename=filename)
            s_attachment.save()
            s_attachment.content.save(
                "attachment-{0}.txt".format(s_attachment.pk),
                ContentFile("".join(lines).encode("UTF-8")))
            # Collect this attachment for cleanup in case something goes wrong
            # and we need to rollback the transaction
            self._content_files.append(s_attachment)

    def _translate_result_string(self, result):
        """
        Translate result string used by client-side API to our internal
        database integer representing the same value.
        """
        from dashboard_app.models import TestResult
        return {
            "pass": TestResult.RESULT_PASS,
            "fail": TestResult.RESULT_FAIL,
            "skip": TestResult.RESULT_SKIP,
            "unknown": TestResult.RESULT_UNKNOWN
        }[result]

    def _get_sw_context(self, c_test_run):
        return c_test_run.get("sw_context", {})

    def _get_hw_context(self, c_test_run):
        return c_test_run.get("hw_context", {})


class BundleFormatImporter_1_0_1(BundleFormatImporter_1_0):
    """
    IFormatImporter subclass capable of loading "Dashboard Bundle Format 1.0.1"
    """

    def _get_sw_context(self, c_test_run):
        return c_test_run.get("software_context", {})

    def _get_hw_context(self, c_test_run):
        return c_test_run.get("hardware_context", {})


class BundleFormatImporter_1_1(BundleFormatImporter_1_0_1):
    """
    IFormatImporter subclass capable of loading "Dashboard Bundle Format 1.1"
    """

    def _import_software_context(self, c_test_run, s_test_run):
        """
        Import software context in 1.1 format.

        Note: We're not upcalling super here as the second line importing
        software image name is quite different in the previous format and I did
        not want to create another function for that. Copying the frozen
        implementation from previous format is IMHO cleaner.
        """
        self._import_packages(c_test_run, s_test_run)
        s_test_run.sw_image_desc = self._get_sw_context(c_test_run).get(
                "image", {}).get("name", "")
        self._import_sources(c_test_run, s_test_run)

    def _import_attachments(self, c_test_run, s_test_run):
        """
        Import TestRun.attachments
        """
        for c_attachment in c_test_run.get("attachments", []):
            s_attachment = s_test_run.attachments.create(
                content_filename = c_attachment["pathname"],
                mime_type = c_attachment["mime_type"])
            # Save to get pk
            s_attachment.save()
            content = base64.standard_b64decode(c_attachment["content"])
            s_attachment.content.save(
                "attachment-{0}.txt".format(s_attachment.pk),
                ContentFile(content))

    def _import_sources(self, c_test_run, s_test_run):
        """
        Import TestRun.sources
        """
        from dashboard_app.models import SoftwareSource

        for c_source in self._get_sw_context(c_test_run).get("sources", []):
            s_source, source_created = SoftwareSource.objects.get_or_create(
                project_name = c_source["project_name"], # required by schema
                branch_url = c_source["branch_url"], # required by schema
                branch_vcs = c_source["branch_vcs"], # required by schema
                # required by schema, may be either int or string so upconvert to string
                branch_revision = str(c_source["branch_revision"]),
                # optional
                commit_timestamp = (
                    datetime_extension.from_json(
                        c_source["commit_timestamp"])
                    if "commit_timestamp" in c_source
                    else None)
            )
            if source_created:
                s_source.save()
            s_test_run.sources.add(s_source)


class BundleFormatImporter_1_2(BundleFormatImporter_1_1):
    """
    IFormatImporter subclass capable of loading "Dashboard Bundle Format 1.2"
    """

    def _import_attachments(self, c_test_run, s_test_run):
        """
        Import TestRun.attachments
        """
        for c_attachment in c_test_run.get("attachments", []):
            s_attachment = s_test_run.attachments.create(
                content_filename = c_attachment["pathname"],
                public_url = c_attachment.get("public_url", ""),
                mime_type = c_attachment["mime_type"])
            s_attachment.save()
            if "content" in c_attachment:
                # Content is optional now
                content = base64.standard_b64decode(c_attachment["content"])
                s_attachment.content.save(
                    "attachment-{0}.txt".format(s_attachment.pk),
                    ContentFile(content))


class BundleFormatImporter_1_3(BundleFormatImporter_1_2):
    """
    IFormatImporter subclass capable of loading "Dashboard Bundle Format 1.3"
    """

    def _import_test_run(self, c_test_run, s_bundle):
        from dashboard_app.models import Tag

        s_test_run = super(BundleFormatImporter_1_3, self)._import_test_run(c_test_run, s_bundle)
        self._log('tags')
        for c_tag in c_test_run.get("tags", []):
            s_tag, created = Tag.objects.get_or_create(name=c_tag)
            if created:
                s_tag.save()
            s_test_run.tags.add(s_tag)
        return s_test_run


class BundleDeserializer(object):
    """
    Helper class for de-serializing JSON bundle content into database models
    """

    IMPORTERS = {
        "Dashboard Bundle Format 1.0": BundleFormatImporter_1_0,
        "Dashboard Bundle Format 1.0.1": BundleFormatImporter_1_0_1,
        "Dashboard Bundle Format 1.1": BundleFormatImporter_1_1,
        "Dashboard Bundle Format 1.2": BundleFormatImporter_1_2,
        "Dashboard Bundle Format 1.3": BundleFormatImporter_1_3,
    }

    def deserialize(self, s_bundle, prefer_evolution):
        """
        Deserializes specified Bundle.

        :Discussion:
            This method also handles internal transaction handling.
            All operations performed during bundle deserialization are
            _rolled_back_ if anything fails.

            If prefer_evolution is enabled then the document is first evolved
            to the latest known format and only then imported into the
            database. This operation is currently disabled to ensure that all
            old documents are imported exactly as before. Enabling it should
            be quite safe though as it passes all tests.

        :Exceptions raised:
            linaro_json.ValidationError
                When the document does not match the appropriate schema.
            linaro_dashboard_bundle.errors.DocumentFormatError
                When the document format is not in the known set of formats.
            ValueError
                When the text does not represent a correct JSON document.
        """
        assert s_bundle.is_deserialized is False
        s_bundle.content.open('rb')
        try:
            logging.debug("Loading document")
            fmt, doc = DocumentIO.load(s_bundle.content)
            logging.debug("Document loaded")
            if prefer_evolution:
                logging.debug("Evolving document")
                DocumentEvolution.evolve_document(doc)
                logging.debug("Document evolution complete")
                fmt = doc["format"]
        finally:
            s_bundle.content.close()
        importer = self.IMPORTERS.get(fmt)
        if importer is None:
            raise DocumentFormatError(fmt)
        try:
            logging.debug("Importing document")
            importer().import_document(s_bundle, doc)
            logging.debug("Document import complete")
        except Exception as exc:
            logging.debug("Exception while importing document: %r", exc)
            raise



# Copyright (C) 2010, 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Tests for data reports
"""

from django.core.urlresolvers import reverse
from django_testscenarios.ubertest import TestCase
from mocker import Mocker, expect

from dashboard_app.models import DataReport


class DataReportTests(TestCase):

    def test_template_context_has_API_URL(self):
        mocker = Mocker()
        report = mocker.patch(DataReport())
        expect(report._get_raw_html()).result("{{API_URL}}")
        with mocker:
            observed = report.get_html()
            expected = reverse("dashboard_app.views.dashboard_xml_rpc_handler")
            self.assertEqual(observed, expected)

    def test_template_context_does_not_have_RequestContext_things(self):
        mocker = Mocker()
        report = mocker.patch(DataReport())
        expect(report._get_raw_html()).result("{{MEDIA_URL}}")
        with mocker:
            observed = report.get_html()
            expected = ""
            self.assertEqual(observed, expected)




# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Tests for the Test model
"""

from django.db import IntegrityError
from django.test import TestCase
from django_testscenarios.ubertest import TestCaseWithScenarios

from dashboard_app.models import Test


class TestConstructionTests(TestCaseWithScenarios):

    scenarios = [
        ('simple1', {
            'test_id': 'org.linaro.testheads.android',
            'name': "Android test suite"}),
        ('simple2', {
            'test_id': 'org.mozilla.unit-tests',
            'name': "Mozilla unit test collection"})
    ]

    def test_construction(self):
        test = Test(test_id = self.test_id, name = self.name)
        test.save()
        self.assertEqual(test.test_id, self.test_id)
        self.assertEqual(test.name, self.name)

    def test_test_id_uniqueness(self):
        test = Test(test_id = self.test_id, name = self.name)
        test.save()
        test2 = Test(test_id = self.test_id)
        self.assertRaises(IntegrityError, test2.save)


class TestUnicodeTests(TestCase):

    def test_unicode_for_test_with_id(self):
        """Test.test_id used when Test.name is not set"""
        obj = Test(test_id="org.some_test")
        self.assertEqual(unicode(obj), "org.some_test")

    def test_unicode_for_test_with_name(self):
        """Test.name used when available"""
        obj = Test(name="Some Test")
        self.assertEqual(unicode(obj), "Some Test")

    def test_unicode_for_test_with_id_and_name(self):
        """Test.name takes precedence over Test.test_id"""
        obj = Test(name="Some Test", test_id="org.some_test")
        self.assertEqual(unicode(obj), "Some Test")



# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Unit tests for dashboard_app.views.test_run_list
"""

from django.contrib.auth.models import User, Group
from django.core.urlresolvers import reverse
from django_testscenarios.ubertest import TestCaseWithScenarios

from dashboard_app.tests import fixtures
from dashboard_app.tests.utils import TestClient


class TestRunListViewAnonymousTest(TestCaseWithScenarios):

    scenarios = [
        ('anonymous_stream', {
            'pathname': '/anonymous/',
        }),
        ('anonymous_named_stream', {
            'pathname': '/anonymous/name/',
        }),
        ('public_personal_stream', {
            'pathname': '/public/personal/user/',
        }),
        ('public_personal_named_stream', {
            'pathname': '/public/personal/user/name/',
        }),
        ('public_team_stream', {
            'pathname': '/public/team/group/',
        }),
        ('public_team_named_stream', {
            'pathname': '/public/team/group/name/',
        }),
        ('private_personal_stream', {
            'pathname': '/private/personal/user/',
        }),
        ('private_personal_named_stream', {
            'pathname': '/private/personal/user/name/',
        }),
        ('private_team_stream', {
            'pathname': '/private/team/group/',
        }),
        ('private_team_named_stream', {
            'pathname': '/private/team/group/name/',
        }),
    ]

    def setUp(self):
        super(TestRunListViewAnonymousTest, self).setUp()
        self.bundle_stream = fixtures.create_bundle_stream(self.pathname)
        self.user = None
        self.url = reverse("dashboard_app.views.test_run_list", args=[self.bundle_stream.pathname])

    def test_status_code(self):
        response = self.client.get(self.url)
        if self.bundle_stream.is_accessible_by(self.user):
            self.assertEqual(response.status_code, 200)
        else:
            self.assertEqual(response.status_code, 404)

    def test_template_used(self):
        response = self.client.get(self.url)
        if self.bundle_stream.is_accessible_by(self.user):
            self.assertTemplateUsed(response,
                "dashboard_app/test_run_list.html")
        else:
            self.assertTemplateUsed(response,
                "404.html")


class TestRunListViewAuthorizedTest(TestRunListViewAnonymousTest):

    def setUp(self):
        super(TestRunListViewAuthorizedTest, self).setUp()
        self.client = TestClient()
        self.user = User.objects.get_or_create(username="user")[0]
        self.group = Group.objects.get_or_create(name="group")[0]
        self.user.groups.add(self.group)
        self.client.login_user(self.user)



# Copyright (C) 2011 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.


from xml.sax.handler import ContentHandler
import re


class BaseContentHandler(ContentHandler):

    def _end_text(self):
        """
        Stop collecting text and produce a stripped string with de-duplicated
        whitespace
        """
        full_text = re.sub("\s+", " ", u''.join(self._text)).strip()
        self.text = None
        return full_text
        
    def _start_text(self):
        """
        Start collecting text
        """
        self._text = []
            
    def characters(self, content):
        if isinstance(self._text, list):
            self._text.append(content)

    def startDocument(self):
        # Text can be None or a [] that accumulates all detected text
        self._text = None



# Copyright (C) 2010 Linaro Limited
#
# Author: Deepti B. Kalakeri<deepti.kalakeri@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.


from django_testscenarios.ubertest import (TestCase, TestCaseWithScenarios)
from dashboard_app.models import BundleStream, TestRun
from django.contrib.auth.models import (User, Group)

from dashboard_app.tests.utils import TestClient


class TestRunDetailView(TestCase):

    fixtures = ["test_run_detail.json"] 

    def setUp(self):
        super(TestRunDetailView, self).setUp()
        self.test_run_url = TestRun.objects.get(pk=1).get_absolute_url()

    def testrun_valid_page_view(self):
        response = self.client.get(self.test_run_url)
        self.assertEqual(response.status_code, 200)

    def test_template_used(self):
        response = self.client.get(self.test_run_url)
        self.assertTemplateUsed(response,
                "dashboard_app/test_run_detail.html")

    #def testrun_invalid_page_view(self):
    #    invalid_uuid = "0000000-0000-0000-0000-000000000000" 
    #    invalid_test_run_url = reverse("dashboard_app.views.test_run_detail",
    #                                   args=[invalid_uuid])
    #    response = self.client.get(invalid_test_run_url)
    #    self.assertEqual(response.status_code, 404)


class TestRunViewAuth(TestCaseWithScenarios):

    _USER = "private_owner"
    _GROUP = "private_group"
    _UNRELATED_USER = "unrelated-user"
    fixtures = ["test_run_detail.json"] 

    scenarios = [
        ("anonymous_accessing_private", {
            "accessing_user": None,
            "resource_owner": _USER
        }),
        ("anonymous_accessing_shared", {
            "accessing_user": None, 
            "resource_owner": _GROUP
        }),
        ("unrelated_accessing_private", {
            "accessing_user": _UNRELATED_USER,
            "resource_owner": _USER,
        }),
        ("unrelated_accessing_shared", {
            "accessing_user": _UNRELATED_USER,
            "resource_owner": _GROUP
        }),
    ]

    def setUp(self):
        super(TestRunViewAuth, self).setUp()
        self.test_run_url = TestRun.objects.get(pk=1).get_absolute_url()

        # Set resource ownership to group or user
        bundle_stream = BundleStream.objects.get(pk=1)
        if self.resource_owner == self._GROUP:
            bundle_stream.group = Group.objects.create(name=self._USER)
        elif self.resource_owner == self._USER:
            bundle_stream.user = User.objects.create(username=self._USER)
        bundle_stream.is_public = False
        bundle_stream.is_anonymous = False
        bundle_stream.save()

        if self.accessing_user:
            self.accessing_user = User.objects.get_or_create(username=self.accessing_user)[0]
            self.client = TestClient()
            self.client.login_user(self.accessing_user)
       
    def test_run_unauth_access(self):
        response = self.client.get(self.test_run_url)
        self.assertEqual(response.status_code, 404)



# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
URL mappings for the Dashboard application
"""
from django.conf.urls.defaults import *

from dashboard_app.xmlrpc import legacy_mapper
import linaro_django_xmlrpc.views

urlpatterns = patterns(
    'dashboard_app.views',
    url(r'^$', 'index'),
    url(r'^ajax/bundle-viewer/(?P<pk>[0-9]+)/$', 'ajax_bundle_viewer'),
    url(r'^ajax/attachment-viewer/(?P<pk>[0-9]+)/$', 'ajax_attachment_viewer'),
    url(r'^data-views/$', 'data_view_list'),
    url(r'^data-views/(?P<name>[a-zA-Z0-9-_]+)/$', 'data_view_detail'),
    url(r'^reports/$', 'report_list'),
    url(r'^reports/(?P<name>[a-zA-Z0-9-_]+)/$', 'report_detail'),
    url(r'^tests/$', 'test_list'),
    url(r'^tests/(?P<test_id>[^/]+)/$', 'test_detail'),
    url(r'^xml-rpc/$', linaro_django_xmlrpc.views.handler, 
        name='dashboard_app.views.dashboard_xml_rpc_handler',
        kwargs={
            'mapper': legacy_mapper,
            'help_view': 'dashboard_app.views.dashboard_xml_rpc_help'}),
    url(r'^xml-rpc/help/$', linaro_django_xmlrpc.views.help,
        name='dashboard_app.views.dashboard_xml_rpc_help',
        kwargs={
            'mapper': legacy_mapper,
            'template_name': 'dashboard_app/api.html'}),
    url(r'^streams/$', 'bundle_stream_list'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/$', 'bundle_list'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/$', 'bundle_detail'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/json$', 'bundle_json'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/$', 'test_run_detail'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/attachments$', 'attachment_list'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/attachments/(?P<pk>[0-9]+)/$', 'attachment_detail'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/result/(?P<relative_index>[0-9]+)/$', 'test_result_detail'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/hardware-context/$', 'test_run_hardware_context'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)bundles/(?P<content_sha1>[0-9a-z]+)/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/software-context/$', 'test_run_software_context'),
    url(r'^streams(?P<pathname>/[a-zA-Z0-9/._-]+)test-runs/$', 'test_run_list'),
    url(r'^permalink/test-run/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/$', 'redirect_to_test_run'),
    url(r'^permalink/test-run/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/(?P<trailing>.*)$', 'redirect_to_test_run'),
    url(r'^permalink/test-result/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/(?P<relative_index>[0-9]+)/$', 'redirect_to_test_result'),
    url(r'^permalink/test-result/(?P<analyzer_assigned_uuid>[a-zA-Z0-9-]+)/(?P<relative_index>[0-9]+)/(?P<trailing>.*)$', 'redirect_to_test_result'),
    url(r'^permalink/bundle/(?P<content_sha1>[0-9a-z]+)/$', 'redirect_to_bundle'),
    url(r'^permalink/bundle/(?P<content_sha1>[0-9a-z]+)/(?P<trailing>.*)$', 'redirect_to_bundle'),
    url(r'^image_status/$', 'image_status_list'),
    url(r'^image_status/(?P<rootfs_type>[a-zA-Z0-9_-]+)\+(?P<hwpack_type>[a-zA-Z0-9_-]+)/$', 'image_status_detail'),
    url(r'^image_status/(?P<rootfs_type>[a-zA-Z0-9_-]+)\+(?P<hwpack_type>[a-zA-Z0-9_-]+)/test-history/(?P<test_id>[^/]+)/$', 'image_test_history'),
    url(r'^efforts/$', 'testing_effort_list'),
    url(r'^efforts/(?P<pk>[0-9]+)/$', 'testing_effort_detail'),
    url(r'^efforts/(?P<pk>[0-9]+)/update/$', 'testing_effort_update'),
    url(r'^efforts/(?P<project_identifier>[a-z0-9-]+)/\+new/$', 'testing_effort_create'),
)



"""
Patches for django bugs that affect this package
"""

class PatchDjangoTicket1476(object):
    """
    Patch for bug http://code.djangoproject.com/ticket/1476
    """

    @classmethod
    def apply_if_needed(patch):
        import django
        if django.VERSION[0:3] <= (1, 2, 4):
            patch.apply()

    @classmethod
    def apply(patch):
        from django.utils.decorators import method_decorator
        from django.views.decorators.csrf import csrf_protect

        @method_decorator(csrf_protect)
        def __call__(self, request, *args, **kwargs):
            """
            Main method that does all the hard work, conforming to the Django view
            interface.
            """
            if 'extra_context' in kwargs:
                self.extra_context.update(kwargs['extra_context'])
            current_step = self.determine_step(request, *args, **kwargs)
            self.parse_params(request, *args, **kwargs)

            # Sanity check.
            if current_step >= self.num_steps():
                raise Http404('Step %s does not exist' % current_step)

            # Process the current step. If it's valid, go to the next step or call
            # done(), depending on whether any steps remain.
            if request.method == 'POST':
                form = self.get_form(current_step, request.POST)
            else:
                form = self.get_form(current_step)

            if form.is_valid():
                # Validate all the forms. If any of them fail validation, that
                # must mean the validator relied on some other input, such as
                # an external Web site.

                # It is also possible that validation might fail under certain
                # attack situations: an attacker might be able to bypass previous
                # stages, and generate correct security hashes for all the
                # skipped stages by virtue of:
                #  1) having filled out an identical form which doesn't have the
                #     validation (and does something different at the end),
                #  2) or having filled out a previous version of the same form
                #     which had some validation missing,
                #  3) or previously having filled out the form when they had
                #     more privileges than they do now.
                #
                # Since the hashes only take into account values, and not other
                # other validation the form might do, we must re-do validation
                # now for security reasons.
                previous_form_list = [self.get_form(i, request.POST) for i in range(current_step)]

                for i, f in enumerate(previous_form_list):
                    if request.POST.get("hash_%d" % i, '') != self.security_hash(request, f):
                        return self.render_hash_failure(request, i)

                    if not f.is_valid():
                        return self.render_revalidation_failure(request, i, f)
                    else:
                        self.process_step(request, f, i)

                # Now progress to processing this step:
                self.process_step(request, form, current_step)
                next_step = current_step + 1


                if next_step == self.num_steps():
                    return self.done(request, previous_form_list + [form])
                else:
                    form = self.get_form(next_step)
                    self.step = current_step = next_step

            return self.render(form, request, current_step)

        from django.contrib.formtools.wizard import FormWizard
        FormWizard.__call__ = __call__


class PatchDjangoTicket15155(object):
    """
    Patch for bug http://code.djangoproject.com/ticket/15155
    """

    PROPER_FORMAT = r'(?<!%)%s'

    @classmethod
    def apply_if_needed(patch):
        from django.db.backends.sqlite3 import base
        if base.FORMAT_QMARK_REGEX != patch.PROPER_FORMAT:
            patch.apply()

    @classmethod
    def apply(cls):
        from django.db.backends.sqlite3 import base
        import re
        base.FORMAT_QMARK_REGEX = re.compile(cls.PROPER_FORMAT)


def patch():
    PatchDjangoTicket1476.apply_if_needed()
    PatchDjangoTicket15155.apply_if_needed()



from django import template
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import HtmlFormatter


register = template.Library()


class StylizeNode(template.Node):

    def __init__(self, nodelist, *varlist):
        self.nodelist, self.vlist = (nodelist, varlist)

    def render(self, context):
        style = 'text'
        if len(self.vlist) > 0:
            style = template.resolve_variable(self.vlist[0], context)
        code = self.nodelist.render(context)
        lexer = get_lexer_by_name(style, encoding='UTF-8')
        formatter = HtmlFormatter(cssclass="pygments")
        return highlight(code, lexer, formatter) 


@register.tag
def stylize(parser, token):
    """
    Usage: {% stylize "language" %}...language text...{% endstylize %}
    """
    nodelist = parser.parse(('endstylize',))
    parser.delete_first_token()
    return StylizeNode(nodelist, *token.contents.split()[1:])



"""
Tests for the HardwareDevice model
"""

from django.db import IntegrityError
from django.test import TestCase

from dashboard_app.models import HardwareDevice
from dashboard_app.tests.call_helper import ObjectFactoryMixIn


class HardwareDeviceTestCase(TestCase, ObjectFactoryMixIn):

    class Dummy:
        class HardwareDevice:
            device_type = 'device.cpu'
            description = 'some cpu'

    def test_creation(self):
        dummy, hw_device = self.make_and_get_dummy(HardwareDevice)
        hw_device.save()
        self.assertEqual(hw_device.device_type, dummy.device_type)
        self.assertEqual(hw_device.description, dummy.description)

    def test_attributes(self):
        hw_device = self.make(HardwareDevice)
        hw_device.save()
        hw_device.attributes.create(name="connection-bus", value="usb")
        self.assertEqual(hw_device.attributes.count(), 1)
        attr = hw_device.attributes.get()
        self.assertEqual(attr.name, "connection-bus")
        self.assertEqual(attr.value, "usb")

    def test_attributes_uniqueness(self):
        hw_device = self.make(HardwareDevice)
        hw_device.save()
        hw_device.attributes.create(name="name", value="value")
        self.assertRaises(IntegrityError, hw_device.attributes.create,
                name="name", value="value")

    def test_unicode(self):
        obj = HardwareDevice(description=u"ARM SoC")
        self.assertEqual(unicode(obj), u"ARM SoC")




# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

import os

from lava_server.extension import LavaServerExtension


class DashboardExtension(LavaServerExtension):

    @property
    def app_name(self):
        return "dashboard_app"

    @property
    def name(self):
        return "Dashboard"

    @property
    def main_view_name(self):
        return "dashboard_app.views.index"

    @property
    def front_page_template(self):
        return "dashboard_app/front_page_snippet.html"

    def get_front_page_context(self):
        from dashboard_app.models import DataReport
        return {
            'report_list': DataReport.repository.filter(front_page=True)
        }

    @property
    def description(self):
        return "Validation Dashboard"

    @property
    def version(self):
        import dashboard_app 
        import versiontools
        return versiontools.format_version(dashboard_app.__version__, hint=dashboard_app)

    @property
    def api_class(self):
        from dashboard_app.xmlrpc import DashboardAPI
        return DashboardAPI

    def contribute_to_settings(self, settings_module):
        super(DashboardExtension, self).contribute_to_settings(settings_module)
        settings_module['INSTALLED_APPS'].append("linaro_django_pagination")
        settings_module['MIDDLEWARE_CLASSES'].append(
            'linaro_django_pagination.middleware.PaginationMiddleware')
        root_dir = os.path.normpath(os.path.join(os.path.dirname(__file__), ".."))
        settings_module['DATAVIEW_DIRS'] = [
            os.path.join(root_dir, 'examples/views'),
            os.path.join(root_dir, 'production/views')]
        settings_module['DATAREPORT_DIRS'] = [
            os.path.join(root_dir, 'examples/reports'),
            os.path.join(root_dir, 'production/reports')]
        settings_module['TEMPLATE_CONTEXT_PROCESSORS'].append(
            'dashboard_app.context_processors.dashboard_globals'
        )

    def contribute_to_settings_ex(self, settings_module, settings_object):
        settings_module['DATAVIEW_DIRS'] = settings_object._settings.get(
            "DATAVIEW_DIRS", [])
        settings_module['DATAREPORT_DIRS'] = settings_object._settings.get(
            "DATAREPORT_DIRS", [])

        # Enable constrained dataview database if requested
        if settings_object._settings.get("use_dataview_database"):
            # Copy everything from the default database and append _dataview to user
            # name. The rest is out of scope (making sure it's actually setup
            # properly, having permissions to login, permissions to view proper data)
            settings_module['DATABASES']['dataview'] = dict(settings_module['DATABASES']['default'])
            settings_module['DATABASES']['dataview']['USER'] += "_dataview"



#!/usr/bin/env python
# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages


setup(
    name='lava-dashboard',
    version=":versiontools:dashboard_app:__version__",
    author="Zygmunt Krynicki",
    author_email="zygmunt.krynicki@linaro.org",
    packages=find_packages(),
    license="AGPL",
    description="Validation Dashboard for LAVA Server",
    long_description="""
    Validation Dashboard is a repository for test results.
    """,
    url='https://launchpad.net/lava-dashboard',
    #test_suite='dashboard_app.tests.test_suite',
    entry_points="""
        [lava_server.extensions]
        dashboard=dashboard_app.extension:DashboardExtension
        """,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Testing",
    ],
    install_requires=[
        'Django >= 1.2',
        'django-restricted-resource >= 0.2.6',
        'django-staticfiles == 0.3.4',
        'docutils >= 0.6',
        'lava-server >= 0.6.dev',
        'linaro-dashboard-bundle >= 1.5.2',
        'linaro-django-pagination >= 2.0.2',
        'linaro-django-xmlrpc >= 0.4',
        'linaro-json >= 2.0.1',  # TODO: use json-schema-validator
        'pygments >= 1.2',
        'south >= 0.7.3',
        'versiontools >= 1.8',
    ],
    setup_requires=[
        'versiontools >= 1.8',
    ],
    tests_require=[
        'django-testscenarios >= 0.7.1',
        'mocker >= 1.0',
    ],
    zip_safe=False,
    include_package_data=True
)




# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Unit tests for Dashboard API (XML-RPC interface)
"""
import contextlib
import xmlrpclib

from django.core.urlresolvers import reverse
from django_testscenarios.ubertest import TransactionTestCase

from dashboard_app.models import Bundle, BundleStream
from dashboard_app.tests import fixtures
from dashboard_app.tests.utils import DashboardXMLRPCViewsTestCase
from dashboard_app.xmlrpc import errors


class DashboardAPIStreamsTests(DashboardXMLRPCViewsTestCase):

    scenarios = [
        ('empty', {
            'pathnames': [],
            'expected_response': [],
        }),
        ('anonymous_stream', {
            'pathnames': [
                '/anonymous/',
            ],
            'expected_response': [{
                'bundle_count': 0,
                'user': 'anonymous-stream-owner',
                'group': '',
                'name': '',
                'pathname': '/anonymous/'}],
        }),
        ('public_streams_are_shown', {
            'pathnames': [
                '/public/personal/user/',
                '/public/team/group/',
            ],
            'expected_response': [{
                'bundle_count': 0,
                'user': 'user',
                'group': '',
                'name': '',
                'pathname': '/public/personal/user/',
            }, {
                'bundle_count': 0,
                'user': '',
                'group': 'group',
                'name': '',
                'pathname': '/public/team/group/',
            }],
        }),
        ('private_streams_are_not_shown', {
            'pathnames': [
                '/private/personal/user/',
                '/private/team/group/',
            ],
            'expected_response': [],
        }),
    ]

    def test_streams(self):
        """
        Check that calling streams() returns all the registered
        streams visible to anonymous user.
        """
        with fixtures.created_bundle_streams(self.pathnames):
            response = self.xml_rpc_call('streams')
            self.assertEqual(response, self.expected_response)


class DashboardAPIBundlesTests(DashboardXMLRPCViewsTestCase):

    scenarios = [
        ('empty', {
            'query': '/anonymous/',
            # make one anonymous stream so that we don't get 404 accessing missing one
            'bundle_streams': ['/anonymous/'],
            'bundles': [],
            'expected_results': [],
        }),
        ('several_bundles_we_can_see', {
            'query': '/anonymous/',
            'bundle_streams': [],
            'bundles': [
                ('/anonymous/', 'test1.json', '{"foobar": 5}'),
                ('/anonymous/', 'test2.json', '{"froz": "bot"}'),
            ],
            'expected_results': [{
                'content_filename': 'test1.json',
                'content_sha1': '72996acd68de60c766b60c2ca6f6169f67cdde19',
            }, {
                'content_filename': 'test2.json',
                'content_sha1': '67dd49730d4e3b38b840f3d544d45cad74bcfb09',
            }],
        }),
        ('several_bundles_in_other_stream', {
            'query': '/anonymous/other/',
            'bundle_streams': [],
            'bundles': [
                ('/anonymous/', 'test3.json', '{}'),
                ('/anonymous/other/', 'test4.json', '{"x": true}'),
            ],
            'expected_results': [{
                'content_filename': 'test4.json',
                'content_sha1': 'bac148f29c35811441a7b4746a022b04c65bffc0',
            }],
        }),
    ]

    def test_bundles(self):
        """
        Make a bunch of bundles (all in a public branch) and check that
        they are returned by the XML-RPC request.
        """
        with contextlib.nested(
            fixtures.created_bundle_streams(self.bundle_streams),
            fixtures.created_bundles(self.bundles)
        ):
            results = self.xml_rpc_call('bundles', self.query)
            self.assertEqual(len(results), len(self.expected_results))
            with fixtures.test_loop(zip(results, self.expected_results)) as loop_items:
                for result, expected_result in loop_items:
                    self.assertEqual(
                            result['content_filename'],
                            expected_result['content_filename'])
                    self.assertEqual(
                            result['content_sha1'],
                            expected_result['content_sha1'])


class DashboardAPIBundlesFailureTests(DashboardXMLRPCViewsTestCase):

    scenarios = [
        ('no_such_stream', {
            'bundle_streams': [],
            'query': '/anonymous/',
        }),
        ('no_anonymous_access_to_private_personal_streams', {
            'bundle_streams': [
                '/private/personal/user/',
            ],
            'query': '/private/personal/user/',
        }),
        ('no_anonymous_access_to_private_team_streams', {
            'bundle_streams': [
                '/private/team/group/',
            ],
            'query': '/private/team/group/',
        }),
    ]

    def test_bundles_failure(self):
        with fixtures.created_bundle_streams(self.bundle_streams):
            try:
                self.xml_rpc_call("bundles", self.query)
            except xmlrpclib.Fault as ex:
                self.assertEqual(ex.faultCode, errors.NOT_FOUND)
            else:
                self.fail("Should have raised an exception")


class DashboardAPIGetTests(DashboardXMLRPCViewsTestCase):

    scenarios = [
        ('bundle_we_can_access', {
            'content_sha1': '72996acd68de60c766b60c2ca6f6169f67cdde19',
            'bundles': [
                ('/anonymous/', 'test1.json', '{"foobar": 5}'),
                ('/anonymous/', 'test2.json', '{"froz": "bot"}'),
            ],
            'expected_result': {
                'content_filename': 'test1.json',
                'content': '{"foobar": 5}',
            }
        }),
    ]

    def test_get(self):
        """
        Make a bunch of bundles (all in a public branch) and check that
        we can get them back by calling get()
        """
        with fixtures.created_bundles(self.bundles):
            result = self.xml_rpc_call('get', self.content_sha1)
            self.assertTrue(isinstance(result, dict))
            self.assertEqual(
                    result['content_filename'],
                    self.expected_result['content_filename'])
            self.assertEqual(
                    result['content'],
                    self.expected_result['content'])


class DashboardAPIGetFailureTests(DashboardXMLRPCViewsTestCase):

    scenarios = [
        ('bad_sha1', {
            'content_sha1': '',
        }),
        ('no_access_to_personal_bundles', {
            'bundles': [
                ('/private/personal/bob/', 'test1.json', '{"foobar": 5}'),
            ],
        }),
        ('no_access_to_named_personal_bundles', {
            'bundles': [
                ('/private/personal/bob/some-name/', 'test1.json', '{"foobar": 5}'),
            ],
        }),
        ('no_access_to_team_bundles', {
            'bundles': [
                ('/private/team/members/', 'test1.json', '{"foobar": 5}'),
            ],
        }),
        ('no_access_to_named_team_bundles', {
            'bundles': [
                ('/private/team/members/some-name/', 'test1.json', '{"foobar": 5}'),
            ],
        }),
    ]

    bundles = []
    # SHA1 of the content used in scenarios above
    content_sha1='72996acd68de60c766b60c2ca6f6169f67cdde19'

    def test_get_failure(self):
        with fixtures.created_bundles(self.bundles):
            try:
                self.xml_rpc_call('get', self.content_sha1)
            except xmlrpclib.Fault as ex:
                self.assertEqual(ex.faultCode, errors.NOT_FOUND)
            else:
                self.fail("Should have raised an exception")


class DashboardAPIPutTests(DashboardXMLRPCViewsTestCase):

    scenarios = [
        ('store_to_public_stream', {
            'content': '{"foobar": 5}',
            'content_filename': 'test1.json',
            'pathname': '/anonymous/',
        }),
        ('store_to_public_named_stream', {
            'content': '{"foobar": 5}',
            'content_filename': 'test1.json',
            'pathname': '/anonymous/some-name/',
        }),
    ]

    def setUp(self):
        super(DashboardAPIPutTests, self).setUp()
        self.bundle_stream = fixtures.create_bundle_stream(self.pathname)
        self.bundle = None

    def tearDown(self):
        if self.bundle:
            self.bundle.delete_files()
        super(DashboardAPIPutTests, self).tearDown()

    def test_put(self):
        content_sha1 = self.xml_rpc_call(
            "put", self.content, self.content_filename, self.pathname)
        self.bundle = Bundle.objects.get(content_sha1=content_sha1)
        self.assertEqual(self.bundle.content_sha1, content_sha1)
        self.assertEqual(self.bundle.content.read(), self.content)
        self.assertEqual(self.bundle.content_filename, self.content_filename)
        self.assertEqual(self.bundle.bundle_stream.pathname, self.pathname)


class DashboardAPIPutFailureTests(DashboardXMLRPCViewsTestCase):
   
    scenarios = [
        ('store_to_personal_stream', {
            'content': '{"foobar": 5}',
            'content_filename': 'test1.json',
            'pathname': '/private/personal/user/',
            'faultCode': errors.NOT_FOUND,
            }),
        ('store_to_named_personal_stream', {
            'content': '{"foobar": 5}',
            'content_filename': 'test1.json',
            'pathname': '/private/personal/user/name/',
            'faultCode': errors.NOT_FOUND,
            }),
        ('store_to_team_stream', {
            'content': '{"foobar": 5}',
            'content_filename': 'test1.json',
            'pathname': '/private/team/group/',
            'faultCode': errors.NOT_FOUND,
            }),
        ('store_to_named_team_stream', {
            'content': '{"foobar": 5}',
            'content_filename': 'test1.json',
            'pathname': '/private/team/group/name/',
            'faultCode': errors.NOT_FOUND,
            }),
        ('store_to_missing_stream', {
            'content': '{"foobar": 5}',
            'content_filename': 'test1.json',
            'pathname': '/anonymous/',
            'faultCode': errors.NOT_FOUND,
            'do_not_create': True,
            }),
        ]

    def test_put_failure(self):
        with contextlib.nested(
            fixtures.created_bundle_streams(
                [] if getattr(self, 'do_not_create', False) else [self.pathname]),
            fixtures.created_bundles(getattr(self, 'bundles', []))
        ):
            try:
                self.xml_rpc_call(
                    "put", self.content, self.content_filename, self.pathname)
            except xmlrpclib.Fault as ex:
                self.assertEqual(ex.faultCode, self.faultCode)
            else:
                self.fail("Should have raised an exception")


class DashboardAPIPutFailureTransactionTests(TransactionTestCase):

    def setUp(self):
        super(DashboardAPIPutFailureTransactionTests, self).setUp()
        self.endpoint_path = reverse(
            "dashboard_app.views.dashboard_xml_rpc_handler")
        self.content_sha1 = None

    def tearDown(self):
        if self.content_sha1:
            Bundle.objects.get(content_sha1=self.content_sha1).delete_files()
        super(DashboardAPIPutFailureTransactionTests, self).tearDown()

    def xml_rpc_call(self, method, *args):
        request_body = xmlrpclib.dumps(tuple(args), methodname=method)
        response = self.client.post(self.endpoint_path,
                request_body, "text/xml")
        return xmlrpclib.loads(response.content)[0][0]

    def test_deserialize_failure_does_not_kill_the_bundle(self):
        _pathname = '/anonymous/'
        _content = '"unterminated string'
        _content_filename = 'bad.json'
        # The test goes via the xml-rpc interface to use views calling the
        # put() API directly will never trigger transactions handling
        with fixtures.created_bundle_streams([_pathname]):
            self.content_sha1 = self.xml_rpc_call(
                "put", _content, _content_filename, _pathname)
            self.assertEqual(Bundle.objects.all().count(), 1)

    def test_put_duplicate(self):
        _bundles = [('/anonymous/', 'test1.json', '{"foobar": 5}')]
        _content = '{"foobar": 5}'
        _content_filename = 'test1.json'
        _pathname = '/anonymous/'
        with fixtures.created_bundles(_bundles):
            try:
                self.content_sha1 = self.xml_rpc_call(
                    "put", _content, _content_filename, _pathname)
            except xmlrpclib.Fault as ex:
                self.assertEqual(ex.faultCode, errors.CONFLICT)
            else:
                self.fail("Should have raised an exception")


class DashboardAPIMakeStreamTests(DashboardXMLRPCViewsTestCase):

    scenarios = [
        ('toplevel_anonymous/', {
            'pathname': '/anonymous/',
            'name': '',
        }),
        ('anonymous_with_slug', {
            'pathname': '/anonymous/some-name/',
            'name': '',
        }),
        ('anonymous_with_slug_and_name', {
            'pathname': '/anonymous/some-name/',
            'name': 'nonempty',
        }),
    ]

    def setUp(self):
        super(DashboardAPIMakeStreamTests, self).setUp()
        self.response = self.xml_rpc_call("make_stream", self.pathname, self.name)

    def test_response_is_pathname(self):
        self.assertEqual(self.response, self.pathname)

    def test_pathname_is_created(self):
        self.assertEqual(BundleStream.objects.filter(pathname=self.pathname).count(), 1)

    def test_pathname_has_name(self):
        bundle_stream = BundleStream.objects.get(pathname=self.pathname)
        self.assertEqual(self.name, bundle_stream.name)


class DashboardAPIMakeStreamFailureTests(DashboardXMLRPCViewsTestCase):

    _NAME = ""

    scenarios = [
        ('public_personal', {
            'pathname': '/public/personal/NAME/',
        }),
        ('private_personal', {
            'pathname': '/private/personal/NAME/',
        }),
        ('public_team', {
            'pathname': '/public/team/NAME/',
        }),
        ('private_team', {
            'pathname': '/private/team/NAME/',
        }),
        ('public_personal_with_slug', {
            'pathname': '/public/personal/NAME/SLUG/',
        }),
        ('private_personal_with_slug', {
            'pathname': '/private/personal/NAME/SLUG/',
        }),
        ('public_team_with_slug', {
            'pathname': '/public/team/NAME/SLUG/',
        }),
        ('private_team_with_slug', {
            'pathname': '/private/team/NAME/SLUG/',
        }),
        ('bogus', {
            'pathname': '/bogus/pathname/'
        }),
    ]

    def test_nonanonymous_streams(self):
        try:
            self.xml_rpc_call("make_stream", self.pathname, self._NAME)
        except xmlrpclib.Fault as ex:
            self.assertEqual(ex.faultCode, errors.FORBIDDEN)
        else:
            self.fail("Should have raised an exception")



"""
Django-specific test utilities
"""
import xmlrpclib

from django.conf import settings
from django.contrib.auth import login
from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.test.client import Client
from django.utils.importlib import import_module

from django_testscenarios.ubertest import (TestCase, TestCaseWithScenarios)


class CSRFTestCase(TestCase):
    """
    Subclass of django's own test.TestCase that allows to interact with cross
    site request forgery protection that is disabled by the regular
    TestCase.

    TODO: Remove this
    """

    def setUp(self):
        super(CSRFTestCase, self).setUp()
        self.client = Client(enforce_csrf_checks=True)


class TestClient(Client):

    def login_user(self, user):
        """
        Login as specified user, does not depend on auth backend (hopefully)

        This is based on Client.login() with a small hack that does not
        require the call to authenticate()
        """
        if not 'django.contrib.sessions' in settings.INSTALLED_APPS:
            raise EnvironmentError("Unable to login without django.contrib.sessions in INSTALLED_APPS")
        user.backend = "%s.%s" % ("django.contrib.auth.backends",
                                  "ModelBackend")
        engine = import_module(settings.SESSION_ENGINE)

        # Create a fake request to store login details.
        request = HttpRequest()
        if self.session:
            request.session = self.session
        else:
            request.session = engine.SessionStore()
        login(request, user)

        # Set the cookie to represent the session.
        session_cookie = settings.SESSION_COOKIE_NAME
        self.cookies[session_cookie] = request.session.session_key
        cookie_data = {
            'max-age': None,
            'path': '/',
            'domain': settings.SESSION_COOKIE_DOMAIN,
            'secure': settings.SESSION_COOKIE_SECURE or None,
            'expires': None,
        }
        self.cookies[session_cookie].update(cookie_data)

        # Save the session values.
        request.session.save()


class DashboardViewsTestCase(TestCaseWithScenarios):
    """
    TODO: Remove this
    """


class DashboardXMLRPCViewsTestCase(TestCaseWithScenarios):
    """
    Helper base class for doing XML-RPC requests
    """

    def setUp(self):
        super(DashboardXMLRPCViewsTestCase, self).setUp()
        self.endpoint_path = reverse(
            "dashboard_app.views.dashboard_xml_rpc_handler")

    def xml_rpc_call(self, method, *args):
        request_body = xmlrpclib.dumps(tuple(args), methodname=method)
        response = self.client.post(self.endpoint_path,
                request_body, "text/xml")
        return xmlrpclib.loads(response.content)[0][0]



from django import forms
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _

from lava_projects.models import Project
from dashboard_app.models import TestingEffort


class TestingEffortForm(forms.Form):

    name = forms.CharField(
        label=_(u"Name"),
        max_length=100)

    description = forms.CharField(
        required=False,
        widget=forms.widgets.Textarea(),
        label=_(u"Description"),
        help_text=_(u"Description of this testing effort"))

    tags = forms.CharField(
        required=False,
        label=_(u"Tags"),
        max_length=1024,
        help_text=_(u"Tags, separated by whitespace or commas"))



# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of Launch Control.
#
# Launch Control is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License version 3
# as published by the Free Software Foundation
#
# Launch Control is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with Launch Control.  If not, see <http://www.gnu.org/licenses/>.

"""
Tests for the TestRun model
"""

import datetime

from django_testscenarios.ubertest import TestCase

from dashboard_app.tests import fixtures
from dashboard_app.models import Test, TestRun


class TestRunTests(TestCase):

    _TEST_ID = "test_id"
    _BUNDLE_PATHNAME = "/anonymous/"
    _BUNDLE_CONTENT_FILENAME = "bundle.txt"
    _BUNDLE_CONTENT = "content not relevant"

    def test_construction(self):
        test = Test.objects.create(test_id=self._TEST_ID)
        analyzer_assigned_uuid = '9695b58e-bfe9-11df-a9a4-002163936223'
        analyzer_assigned_date = datetime.datetime(2010, 9, 14, 12, 20, 00)
        time_check_performed = False
        spec = [(self._BUNDLE_PATHNAME, self._BUNDLE_CONTENT_FILENAME,
                self._BUNDLE_CONTENT)]
        with fixtures.created_bundles(spec) as bundles:
            test_run = TestRun(
                bundle = bundles[0],
                test = test,
                time_check_performed=time_check_performed,
                analyzer_assigned_uuid = analyzer_assigned_uuid,
                analyzer_assigned_date = analyzer_assigned_date,
            )
            test_run.save()
            self.assertEqual(test_run.bundle, bundles[0])
            self.assertEqual(test_run.time_check_performed, time_check_performed)
            self.assertEqual(test_run.test, test)
            self.assertEqual(test_run.analyzer_assigned_uuid,
                             analyzer_assigned_uuid)

    def test_unicode(self):
        obj = TestRun(analyzer_assigned_uuid="0" * 16)
        self.assertIn(obj.analyzer_assigned_uuid, unicode(obj))
